#ifndef ELF_RELOC
#error "ELF_RELOC must be defined"
#endif

// No relocation
ELF_RELOC(R_BPF_NONE,        0)
ELF_RELOC(R_BPF_64_64,       1)
ELF_RELOC(R_BPF_64_ABS64,    2)
ELF_RELOC(R_BPF_64_ABS32,    3)
ELF_RELOC(R_BPF_64_NODYLD32, 4)
ELF_RELOC(R_BPF_64_32,      10)
