/*
 * THIS FILE AUTOMATICALLY GENERATED.  DO NOT EDIT.
 *
 * generated from:
 *	OpenBSD: eisadevs,v 1.20 2004/05/05 05:33:56 mickey Exp 
 */

/*
 * Copyright (c) 1995, 1996 Christopher G. Demetriou
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Christopher G. Demetriou
 *	for the NetBSD Project.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

static const struct eisa_knowndev eisa_knowndevs[] = {
	{
	    0,
	    "ABP0510",
	    EISA_PRODUCT_ABP0510,
	},
	{
	    0,
	    "ABP0540",
	    EISA_PRODUCT_ABP0540,
	},
	{
	    0,
	    "ABP7401",
	    EISA_PRODUCT_ABP7401,
	},
	{
	    0,
	    "ABP7501",
	    EISA_PRODUCT_ABP7501,
	},
	{
	    0,
	    "ACC1200",
	    EISA_PRODUCT_ACC1200,
	},
	{
	    0,
	    "ACC120A",
	    EISA_PRODUCT_ACC120A,
	},
	{
	    0,
	    "ACC1650",
	    EISA_PRODUCT_ACC1650,
	},
	{
	    0,
	    "ACC1660",
	    EISA_PRODUCT_ACC1660,
	},
	{
	    0,
	    "ACE1010",
	    EISA_PRODUCT_ACE1010,
	},
	{
	    0,
	    "ACE2010",
	    EISA_PRODUCT_ACE2010,
	},
	{
	    0,
	    "ACE3010",
	    EISA_PRODUCT_ACE3010,
	},
	{
	    0,
	    "ACE3030",
	    EISA_PRODUCT_ACE3030,
	},
	{
	    0,
	    "ACE4010",
	    EISA_PRODUCT_ACE4010,
	},
	{
	    0,
	    "ACE5010",
	    EISA_PRODUCT_ACE5010,
	},
	{
	    0,
	    "ACE6010",
	    EISA_PRODUCT_ACE6010,
	},
	{
	    0,
	    "ACE7010",
	    EISA_PRODUCT_ACE7010,
	},
	{
	    0,
	    "ACR1201",
	    EISA_PRODUCT_ACR1201,
	},
	{
	    0,
	    "ACR1211",
	    EISA_PRODUCT_ACR1211,
	},
	{
	    0,
	    "ACR1341",
	    EISA_PRODUCT_ACR1341,
	},
	{
	    0,
	    "ACR1351",
	    EISA_PRODUCT_ACR1351,
	},
	{
	    0,
	    "ACR1361",
	    EISA_PRODUCT_ACR1361,
	},
	{
	    0,
	    "ACR1371",
	    EISA_PRODUCT_ACR1371,
	},
	{
	    0,
	    "ACR1381",
	    EISA_PRODUCT_ACR1381,
	},
	{
	    0,
	    "ACR1391",
	    EISA_PRODUCT_ACR1391,
	},
	{
	    0,
	    "ACR1581",
	    EISA_PRODUCT_ACR1581,
	},
	{
	    0,
	    "ACR1591",
	    EISA_PRODUCT_ACR1591,
	},
	{
	    0,
	    "ACR15A1",
	    EISA_PRODUCT_ACR15A1,
	},
	{
	    0,
	    "ACR15B1",
	    EISA_PRODUCT_ACR15B1,
	},
	{
	    0,
	    "ACR1701",
	    EISA_PRODUCT_ACR1701,
	},
	{
	    0,
	    "ACR1711",
	    EISA_PRODUCT_ACR1711,
	},
	{
	    0,
	    "ACR1801",
	    EISA_PRODUCT_ACR1801,
	},
	{
	    0,
	    "ACR3211",
	    EISA_PRODUCT_ACR3211,
	},
	{
	    0,
	    "ACR3221",
	    EISA_PRODUCT_ACR3221,
	},
	{
	    0,
	    "ACR3231",
	    EISA_PRODUCT_ACR3231,
	},
	{
	    0,
	    "ACR3241",
	    EISA_PRODUCT_ACR3241,
	},
	{
	    0,
	    "ACR3261",
	    EISA_PRODUCT_ACR3261,
	},
	{
	    0,
	    "ACR3271",
	    EISA_PRODUCT_ACR3271,
	},
	{
	    0,
	    "ACR3281",
	    EISA_PRODUCT_ACR3281,
	},
	{
	    0,
	    "ACR3291",
	    EISA_PRODUCT_ACR3291,
	},
	{
	    0,
	    "ACR4509",
	    EISA_PRODUCT_ACR4509,
	},
	{
	    0,
	    "ADI0001",
	    EISA_PRODUCT_ADI0001,
	},
	{
	    0,
	    "ADP0000",
	    EISA_PRODUCT_ADP0000,
	},
	{
	    0,
	    "ADP0001",
	    EISA_PRODUCT_ADP0001,
	},
	{
	    0,
	    "ADP0002",
	    EISA_PRODUCT_ADP0002,
	},
	{
	    0,
	    "ADP0100",
	    EISA_PRODUCT_ADP0100,
	},
	{
	    0,
	    "ADP0200",
	    EISA_PRODUCT_ADP0200,
	},
	{
	    0,
	    "ADP0400",
	    EISA_PRODUCT_ADP0400,
	},
	{
	    0,
	    "ADP7770",
	    EISA_PRODUCT_ADP7770,
	},
	{
	    0,
	    "ADP7771",
	    EISA_PRODUCT_ADP7771,
	},
	{
	    0,
	    "ADP7756",
	    EISA_PRODUCT_ADP7756,
	},
	{
	    0,
	    "ADP7757",
	    EISA_PRODUCT_ADP7757,
	},
	{
	    0,
	    "AEI0401",
	    EISA_PRODUCT_AEI0401,
	},
	{
	    0,
	    "AEO0301",
	    EISA_PRODUCT_AEO0301,
	},
	{
	    0,
	    "AIM0002",
	    EISA_PRODUCT_AIM0002,
	},
	{
	    0,
	    "AIR0101",
	    EISA_PRODUCT_AIR0101,
	},
	{
	    0,
	    "AIR0103",
	    EISA_PRODUCT_AIR0103,
	},
	{
	    0,
	    "AIR0201",
	    EISA_PRODUCT_AIR0201,
	},
	{
	    0,
	    "AIR0301",
	    EISA_PRODUCT_AIR0301,
	},
	{
	    0,
	    "AIR0401",
	    EISA_PRODUCT_AIR0401,
	},
	{
	    0,
	    "AIR0501",
	    EISA_PRODUCT_AIR0501,
	},
	{
	    0,
	    "AIR0601",
	    EISA_PRODUCT_AIR0601,
	},
	{
	    0,
	    "AIR0701",
	    EISA_PRODUCT_AIR0701,
	},
	{
	    0,
	    "AIR0702",
	    EISA_PRODUCT_AIR0702,
	},
	{
	    0,
	    "AIR0901",
	    EISA_PRODUCT_AIR0901,
	},
	{
	    0,
	    "AIR1001",
	    EISA_PRODUCT_AIR1001,
	},
	{
	    0,
	    "AIR2001",
	    EISA_PRODUCT_AIR2001,
	},
	{
	    0,
	    "AIR2101",
	    EISA_PRODUCT_AIR2101,
	},
	{
	    0,
	    "AIR3001",
	    EISA_PRODUCT_AIR3001,
	},
	{
	    0,
	    "AIR3101",
	    EISA_PRODUCT_AIR3101,
	},
	{
	    0,
	    "ALR0001",
	    EISA_PRODUCT_ALR0001,
	},
	{
	    0,
	    "ALR0041",
	    EISA_PRODUCT_ALR0041,
	},
	{
	    0,
	    "ALR0181",
	    EISA_PRODUCT_ALR0181,
	},
	{
	    0,
	    "ALR0241",
	    EISA_PRODUCT_ALR0241,
	},
	{
	    0,
	    "ALR0341",
	    EISA_PRODUCT_ALR0341,
	},
	{
	    0,
	    "ALR3000",
	    EISA_PRODUCT_ALR3000,
	},
	{
	    0,
	    "ALR3010",
	    EISA_PRODUCT_ALR3010,
	},
	{
	    0,
	    "ALR3023",
	    EISA_PRODUCT_ALR3023,
	},
	{
	    0,
	    "ALR8580",
	    EISA_PRODUCT_ALR8580,
	},
	{
	    0,
	    "ALRA0C1",
	    EISA_PRODUCT_ALRA0C1,
	},
	{
	    0,
	    "ALRA301",
	    EISA_PRODUCT_ALRA301,
	},
	{
	    0,
	    "ALRA311",
	    EISA_PRODUCT_ALRA311,
	},
	{
	    0,
	    "ALRB0A0",
	    EISA_PRODUCT_ALRB0A0,
	},
	{
	    0,
	    "ALRB0B0",
	    EISA_PRODUCT_ALRB0B0,
	},
	{
	    0,
	    "AMI15C1",
	    EISA_PRODUCT_AMI15C1,
	},
	{
	    0,
	    "AMI15D1",
	    EISA_PRODUCT_AMI15D1,
	},
	{
	    0,
	    "AMI15E1",
	    EISA_PRODUCT_AMI15E1,
	},
	{
	    0,
	    "AMI16B1",
	    EISA_PRODUCT_AMI16B1,
	},
	{
	    0,
	    "AMI2509",
	    EISA_PRODUCT_AMI2509,
	},
	{
	    0,
	    "AMI25B1",
	    EISA_PRODUCT_AMI25B1,
	},
	{
	    0,
	    "AMI28A1",
	    EISA_PRODUCT_AMI28A1,
	},
	{
	    0,
	    "AMI44C1",
	    EISA_PRODUCT_AMI44C1,
	},
	{
	    0,
	    "AMI44D2",
	    EISA_PRODUCT_AMI44D2,
	},
	{
	    0,
	    "AMI4801",
	    EISA_PRODUCT_AMI4801,
	},
	{
	    0,
	    "AMI68B1",
	    EISA_PRODUCT_AMI68B1,
	},
	{
	    0,
	    "APS0101",
	    EISA_PRODUCT_APS0101,
	},
	{
	    0,
	    "APS0102",
	    EISA_PRODUCT_APS0102,
	},
	{
	    0,
	    "APS0103",
	    EISA_PRODUCT_APS0103,
	},
	{
	    0,
	    "ARC0010",
	    EISA_PRODUCT_ARC0010,
	},
	{
	    0,
	    "ARC0020",
	    EISA_PRODUCT_ARC0020,
	},
	{
	    0,
	    "ASU0100",
	    EISA_PRODUCT_ASU0100,
	},
	{
	    0,
	    "ASU0500",
	    EISA_PRODUCT_ASU0500,
	},
	{
	    0,
	    "ASU4001",
	    EISA_PRODUCT_ASU4001,
	},
	{
	    0,
	    "ASU4101",
	    EISA_PRODUCT_ASU4101,
	},
	{
	    0,
	    "ASU4201",
	    EISA_PRODUCT_ASU4201,
	},
	{
	    0,
	    "ASU4301",
	    EISA_PRODUCT_ASU4301,
	},
	{
	    0,
	    "ASU4501",
	    EISA_PRODUCT_ASU4501,
	},
	{
	    0,
	    "ASU4701",
	    EISA_PRODUCT_ASU4701,
	},
	{
	    0,
	    "ASU4901",
	    EISA_PRODUCT_ASU4901,
	},
	{
	    0,
	    "ASU5101",
	    EISA_PRODUCT_ASU5101,
	},
	{
	    0,
	    "ASU5201",
	    EISA_PRODUCT_ASU5201,
	},
	{
	    0,
	    "ATI1500",
	    EISA_PRODUCT_ATI1500,
	},
	{
	    0,
	    "ATI1700",
	    EISA_PRODUCT_ATI1700,
	},
	{
	    0,
	    "ATI4400",
	    EISA_PRODUCT_ATI4400,
	},
	{
	    0,
	    "ATI4402",
	    EISA_PRODUCT_ATI4402,
	},
	{
	    0,
	    "ATI4410",
	    EISA_PRODUCT_ATI4410,
	},
	{
	    0,
	    "ATI4420",
	    EISA_PRODUCT_ATI4420,
	},
	{
	    0,
	    "ATI4430",
	    EISA_PRODUCT_ATI4430,
	},
	{
	    0,
	    "ATT2402",
	    EISA_PRODUCT_ATT2402,
	},
	{
	    0,
	    "ATT2404",
	    EISA_PRODUCT_ATT2404,
	},
	{
	    0,
	    "AVI2E01",
	    EISA_PRODUCT_AVI2E01,
	},
	{
	    0,
	    "AVM0001",
	    EISA_PRODUCT_AVM0001,
	},
	{
	    0,
	    "BAN0440",
	    EISA_PRODUCT_BAN0440,
	},
	{
	    0,
	    "BAN0670",
	    EISA_PRODUCT_BAN0670,
	},
	{
	    0,
	    "BAN0680",
	    EISA_PRODUCT_BAN0680,
	},
	{
	    0,
	    "BUS4201",
	    EISA_PRODUCT_BUS4201,
	},
	{
	    0,
	    "BUS4202",
	    EISA_PRODUCT_BUS4202,
	},
	{
	    0,
	    "BUS6001",
	    EISA_PRODUCT_BUS6001,
	},
	{
	    0,
	    "BUS6301",
	    EISA_PRODUCT_BUS6301,
	},
	{
	    0,
	    "CCI0000",
	    EISA_PRODUCT_CCI0000,
	},
	{
	    0,
	    "CCI1001",
	    EISA_PRODUCT_CCI1001,
	},
	{
	    0,
	    "CCI2001",
	    EISA_PRODUCT_CCI2001,
	},
	{
	    0,
	    "CCI3001",
	    EISA_PRODUCT_CCI3001,
	},
	{
	    0,
	    "CCI3009",
	    EISA_PRODUCT_CCI3009,
	},
	{
	    0,
	    "CCI4000",
	    EISA_PRODUCT_CCI4000,
	},
	{
	    0,
	    "CCI4001",
	    EISA_PRODUCT_CCI4001,
	},
	{
	    0,
	    "CCI5001",
	    EISA_PRODUCT_CCI5001,
	},
	{
	    0,
	    "CCI6001",
	    EISA_PRODUCT_CCI6001,
	},
	{
	    0,
	    "CCI7001",
	    EISA_PRODUCT_CCI7001,
	},
	{
	    0,
	    "CCI8001",
	    EISA_PRODUCT_CCI8001,
	},
	{
	    0,
	    "CHAA041",
	    EISA_PRODUCT_CHAA041,
	},
	{
	    0,
	    "CHAA081",
	    EISA_PRODUCT_CHAA081,
	},
	{
	    0,
	    "CHAA091",
	    EISA_PRODUCT_CHAA091,
	},
	{
	    0,
	    "CHAA161",
	    EISA_PRODUCT_CHAA161,
	},
	{
	    0,
	    "CHAA171",
	    EISA_PRODUCT_CHAA171,
	},
	{
	    0,
	    "CMD0003",
	    EISA_PRODUCT_CMD0003,
	},
	{
	    0,
	    "CNT2000",
	    EISA_PRODUCT_CNT2000,
	},
	{
	    0,
	    "COG5000",
	    EISA_PRODUCT_COG5000,
	},
	{
	    0,
	    "COG7002",
	    EISA_PRODUCT_COG7002,
	},
	{
	    0,
	    "COG9002",
	    EISA_PRODUCT_COG9002,
	},
	{
	    0,
	    "CPQ0101",
	    EISA_PRODUCT_CPQ0101,
	},
	{
	    0,
	    "CPQ0109",
	    EISA_PRODUCT_CPQ0109,
	},
	{
	    0,
	    "CPQ0401",
	    EISA_PRODUCT_CPQ0401,
	},
	{
	    0,
	    "CPQ0501",
	    EISA_PRODUCT_CPQ0501,
	},
	{
	    0,
	    "CPQ0509",
	    EISA_PRODUCT_CPQ0509,
	},
	{
	    0,
	    "CPQ0511",
	    EISA_PRODUCT_CPQ0511,
	},
	{
	    0,
	    "CPQ0601",
	    EISA_PRODUCT_CPQ0601,
	},
	{
	    0,
	    "CPQ0609",
	    EISA_PRODUCT_CPQ0609,
	},
	{
	    0,
	    "CPQ0611",
	    EISA_PRODUCT_CPQ0611,
	},
	{
	    0,
	    "CPQ0621",
	    EISA_PRODUCT_CPQ0621,
	},
	{
	    0,
	    "CPQ0629",
	    EISA_PRODUCT_CPQ0629,
	},
	{
	    0,
	    "CPQ0631",
	    EISA_PRODUCT_CPQ0631,
	},
	{
	    0,
	    "CPQ0639",
	    EISA_PRODUCT_CPQ0639,
	},
	{
	    0,
	    "CPQ1001",
	    EISA_PRODUCT_CPQ1001,
	},
	{
	    0,
	    "CPQ1009",
	    EISA_PRODUCT_CPQ1009,
	},
	{
	    0,
	    "CPQ1201",
	    EISA_PRODUCT_CPQ1201,
	},
	{
	    0,
	    "CPQ1301",
	    EISA_PRODUCT_CPQ1301,
	},
	{
	    0,
	    "CPQ1401",
	    EISA_PRODUCT_CPQ1401,
	},
	{
	    0,
	    "CPQ1409",
	    EISA_PRODUCT_CPQ1409,
	},
	{
	    0,
	    "CPQ1501",
	    EISA_PRODUCT_CPQ1501,
	},
	{
	    0,
	    "CPQ1509",
	    EISA_PRODUCT_CPQ1509,
	},
	{
	    0,
	    "CPQ1519",
	    EISA_PRODUCT_CPQ1519,
	},
	{
	    0,
	    "CPQ3001",
	    EISA_PRODUCT_CPQ3001,
	},
	{
	    0,
	    "CPQ3011",
	    EISA_PRODUCT_CPQ3011,
	},
	{
	    0,
	    "CPQ3021",
	    EISA_PRODUCT_CPQ3021,
	},
	{
	    0,
	    "CPQ3111",
	    EISA_PRODUCT_CPQ3111,
	},
	{
	    0,
	    "CPQ3112",
	    EISA_PRODUCT_CPQ3112,
	},
	{
	    0,
	    "CPQ3121",
	    EISA_PRODUCT_CPQ3121,
	},
	{
	    0,
	    "CPQ3122",
	    EISA_PRODUCT_CPQ3122,
	},
	{
	    0,
	    "CPQ4001",
	    EISA_PRODUCT_CPQ4001,
	},
	{
	    0,
	    "CPQ4002",
	    EISA_PRODUCT_CPQ4002,
	},
	{
	    0,
	    "CPQ4010",
	    EISA_PRODUCT_CPQ4010,
	},
	{
	    0,
	    "CPQ4020",
	    EISA_PRODUCT_CPQ4020,
	},
	{
	    0,
	    "CPQ4300",
	    EISA_PRODUCT_CPQ4300,
	},
	{
	    0,
	    "CPQ4401",
	    EISA_PRODUCT_CPQ4401,
	},
	{
	    0,
	    "CPQ4410",
	    EISA_PRODUCT_CPQ4410,
	},
	{
	    0,
	    "CPQ4411",
	    EISA_PRODUCT_CPQ4411,
	},
	{
	    0,
	    "CPQ4420",
	    EISA_PRODUCT_CPQ4420,
	},
	{
	    0,
	    "CPQ5000",
	    EISA_PRODUCT_CPQ5000,
	},
	{
	    0,
	    "CPQ5281",
	    EISA_PRODUCT_CPQ5281,
	},
	{
	    0,
	    "CPQ5282",
	    EISA_PRODUCT_CPQ5282,
	},
	{
	    0,
	    "CPQ5287",
	    EISA_PRODUCT_CPQ5287,
	},
	{
	    0,
	    "CPQ528F",
	    EISA_PRODUCT_CPQ528F,
	},
	{
	    0,
	    "CPQ5900",
	    EISA_PRODUCT_CPQ5900,
	},
	{
	    0,
	    "CPQ5A00",
	    EISA_PRODUCT_CPQ5A00,
	},
	{
	    0,
	    "CPQ5B00",
	    EISA_PRODUCT_CPQ5B00,
	},
	{
	    0,
	    "CPQ5C00",
	    EISA_PRODUCT_CPQ5C00,
	},
	{
	    0,
	    "CPQ6000",
	    EISA_PRODUCT_CPQ6000,
	},
	{
	    0,
	    "CPQ6001",
	    EISA_PRODUCT_CPQ6001,
	},
	{
	    0,
	    "CPQ6100",
	    EISA_PRODUCT_CPQ6100,
	},
	{
	    0,
	    "CPQ6101",
	    EISA_PRODUCT_CPQ6101,
	},
	{
	    0,
	    "CPQ6200",
	    EISA_PRODUCT_CPQ6200,
	},
	{
	    0,
	    "CPQ7000",
	    EISA_PRODUCT_CPQ7000,
	},
	{
	    0,
	    "CPQ7001",
	    EISA_PRODUCT_CPQ7001,
	},
	{
	    0,
	    "CPQ9004",
	    EISA_PRODUCT_CPQ9004,
	},
	{
	    0,
	    "CPQ9005",
	    EISA_PRODUCT_CPQ9005,
	},
	{
	    0,
	    "CPQ9013",
	    EISA_PRODUCT_CPQ9013,
	},
	{
	    0,
	    "CPQ9014",
	    EISA_PRODUCT_CPQ9014,
	},
	{
	    0,
	    "CPQ9015",
	    EISA_PRODUCT_CPQ9015,
	},
	{
	    0,
	    "CPQ9016",
	    EISA_PRODUCT_CPQ9016,
	},
	{
	    0,
	    "CPQ9017",
	    EISA_PRODUCT_CPQ9017,
	},
	{
	    0,
	    "CPQ9018",
	    EISA_PRODUCT_CPQ9018,
	},
	{
	    0,
	    "CPQ9034",
	    EISA_PRODUCT_CPQ9034,
	},
	{
	    0,
	    "CPQ9035",
	    EISA_PRODUCT_CPQ9035,
	},
	{
	    0,
	    "CPQ9036",
	    EISA_PRODUCT_CPQ9036,
	},
	{
	    0,
	    "CPQ9037",
	    EISA_PRODUCT_CPQ9037,
	},
	{
	    0,
	    "CPQ9038",
	    EISA_PRODUCT_CPQ9038,
	},
	{
	    0,
	    "CPQ903C",
	    EISA_PRODUCT_CPQ903C,
	},
	{
	    0,
	    "CPQ9040",
	    EISA_PRODUCT_CPQ9040,
	},
	{
	    0,
	    "CPQ9041",
	    EISA_PRODUCT_CPQ9041,
	},
	{
	    0,
	    "CPQ9042",
	    EISA_PRODUCT_CPQ9042,
	},
	{
	    0,
	    "CPQ9043",
	    EISA_PRODUCT_CPQ9043,
	},
	{
	    0,
	    "CPQ9044",
	    EISA_PRODUCT_CPQ9044,
	},
	{
	    0,
	    "CPQ9045",
	    EISA_PRODUCT_CPQ9045,
	},
	{
	    0,
	    "CPQ9046",
	    EISA_PRODUCT_CPQ9046,
	},
	{
	    0,
	    "CPQ9047",
	    EISA_PRODUCT_CPQ9047,
	},
	{
	    0,
	    "CPQ9281",
	    EISA_PRODUCT_CPQ9281,
	},
	{
	    0,
	    "CPQ9282",
	    EISA_PRODUCT_CPQ9282,
	},
	{
	    0,
	    "CPQ9287",
	    EISA_PRODUCT_CPQ9287,
	},
	{
	    0,
	    "CPQ928F",
	    EISA_PRODUCT_CPQ928F,
	},
	{
	    0,
	    "CPQ9381",
	    EISA_PRODUCT_CPQ9381,
	},
	{
	    0,
	    "CPQ9382",
	    EISA_PRODUCT_CPQ9382,
	},
	{
	    0,
	    "CPQ9387",
	    EISA_PRODUCT_CPQ9387,
	},
	{
	    0,
	    "CPQ9481",
	    EISA_PRODUCT_CPQ9481,
	},
	{
	    0,
	    "CPQ9482",
	    EISA_PRODUCT_CPQ9482,
	},
	{
	    0,
	    "CPQ9487",
	    EISA_PRODUCT_CPQ9487,
	},
	{
	    0,
	    "CPQ9901",
	    EISA_PRODUCT_CPQ9901,
	},
	{
	    0,
	    "CPQ9902",
	    EISA_PRODUCT_CPQ9902,
	},
	{
	    0,
	    "CPQ9903",
	    EISA_PRODUCT_CPQ9903,
	},
	{
	    0,
	    "CPQ9904",
	    EISA_PRODUCT_CPQ9904,
	},
	{
	    0,
	    "CPQ9905",
	    EISA_PRODUCT_CPQ9905,
	},
	{
	    0,
	    "CPQ9906",
	    EISA_PRODUCT_CPQ9906,
	},
	{
	    0,
	    "CPQ9907",
	    EISA_PRODUCT_CPQ9907,
	},
	{
	    0,
	    "CPQ9908",
	    EISA_PRODUCT_CPQ9908,
	},
	{
	    0,
	    "CPQ9909",
	    EISA_PRODUCT_CPQ9909,
	},
	{
	    0,
	    "CPQ990A",
	    EISA_PRODUCT_CPQ990A,
	},
	{
	    0,
	    "CPQ990B",
	    EISA_PRODUCT_CPQ990B,
	},
	{
	    0,
	    "CPQ990C",
	    EISA_PRODUCT_CPQ990C,
	},
	{
	    0,
	    "CPQ990D",
	    EISA_PRODUCT_CPQ990D,
	},
	{
	    0,
	    "CPQ990E",
	    EISA_PRODUCT_CPQ990E,
	},
	{
	    0,
	    "CPQ990F",
	    EISA_PRODUCT_CPQ990F,
	},
	{
	    0,
	    "CPQ9990",
	    EISA_PRODUCT_CPQ9990,
	},
	{
	    0,
	    "CPQ9991",
	    EISA_PRODUCT_CPQ9991,
	},
	{
	    0,
	    "CPQ9999",
	    EISA_PRODUCT_CPQ9999,
	},
	{
	    0,
	    "CPQ999A",
	    EISA_PRODUCT_CPQ999A,
	},
	{
	    0,
	    "CPQA000",
	    EISA_PRODUCT_CPQA000,
	},
	{
	    0,
	    "CPQA010",
	    EISA_PRODUCT_CPQA010,
	},
	{
	    0,
	    "CPQA020",
	    EISA_PRODUCT_CPQA020,
	},
	{
	    0,
	    "CPQA030",
	    EISA_PRODUCT_CPQA030,
	},
	{
	    0,
	    "CPQF000",
	    EISA_PRODUCT_CPQF000,
	},
	{
	    0,
	    "CPQF100",
	    EISA_PRODUCT_CPQF100,
	},
	{
	    0,
	    "CPQFA0D",
	    EISA_PRODUCT_CPQFA0D,
	},
	{
	    0,
	    "CPQFA0E",
	    EISA_PRODUCT_CPQFA0E,
	},
	{
	    0,
	    "CPQFA0F",
	    EISA_PRODUCT_CPQFA0F,
	},
	{
	    0,
	    "CPQFA1A",
	    EISA_PRODUCT_CPQFA1A,
	},
	{
	    0,
	    "CPQFA1B",
	    EISA_PRODUCT_CPQFA1B,
	},
	{
	    0,
	    "CPQFA1C",
	    EISA_PRODUCT_CPQFA1C,
	},
	{
	    0,
	    "CPQFA1D",
	    EISA_PRODUCT_CPQFA1D,
	},
	{
	    0,
	    "CPQFB03",
	    EISA_PRODUCT_CPQFB03,
	},
	{
	    0,
	    "CPQFB07",
	    EISA_PRODUCT_CPQFB07,
	},
	{
	    0,
	    "CPQFC0B",
	    EISA_PRODUCT_CPQFC0B,
	},
	{
	    0,
	    "CPQFD08",
	    EISA_PRODUCT_CPQFD08,
	},
	{
	    0,
	    "CPQFD13",
	    EISA_PRODUCT_CPQFD13,
	},
	{
	    0,
	    "CPQFD17",
	    EISA_PRODUCT_CPQFD17,
	},
	{
	    0,
	    "CPX0301",
	    EISA_PRODUCT_CPX0301,
	},
	{
	    0,
	    "CRS3203",
	    EISA_PRODUCT_CRS3203,
	},
	{
	    0,
	    "CRS3204",
	    EISA_PRODUCT_CRS3204,
	},
	{
	    0,
	    "CSI0690",
	    EISA_PRODUCT_CSI0690,
	},
	{
	    0,
	    "CUI0000",
	    EISA_PRODUCT_CUI0000,
	},
	{
	    0,
	    "DBI0101",
	    EISA_PRODUCT_DBI0101,
	},
	{
	    0,
	    "DBI0102",
	    EISA_PRODUCT_DBI0102,
	},
	{
	    0,
	    "DBI0201",
	    EISA_PRODUCT_DBI0201,
	},
	{
	    0,
	    "DBI0301",
	    EISA_PRODUCT_DBI0301,
	},
	{
	    0,
	    "DBI0501",
	    EISA_PRODUCT_DBI0501,
	},
	{
	    0,
	    "DBI0601",
	    EISA_PRODUCT_DBI0601,
	},
	{
	    0,
	    "DBI0701",
	    EISA_PRODUCT_DBI0701,
	},
	{
	    0,
	    "DBI0801",
	    EISA_PRODUCT_DBI0801,
	},
	{
	    0,
	    "DBI0901",
	    EISA_PRODUCT_DBI0901,
	},
	{
	    0,
	    "DBI0C01",
	    EISA_PRODUCT_DBI0C01,
	},
	{
	    0,
	    "DBI0D01",
	    EISA_PRODUCT_DBI0D01,
	},
	{
	    0,
	    "DBI0E01",
	    EISA_PRODUCT_DBI0E01,
	},
	{
	    0,
	    "DBI0F01",
	    EISA_PRODUCT_DBI0F01,
	},
	{
	    0,
	    "DBI1001",
	    EISA_PRODUCT_DBI1001,
	},
	{
	    0,
	    "DBI1101",
	    EISA_PRODUCT_DBI1101,
	},
	{
	    0,
	    "DBI1201",
	    EISA_PRODUCT_DBI1201,
	},
	{
	    0,
	    "DEC1011",
	    EISA_PRODUCT_DEC1011,
	},
	{
	    0,
	    "DEC1021",
	    EISA_PRODUCT_DEC1021,
	},
	{
	    0,
	    "DEC1031",
	    EISA_PRODUCT_DEC1031,
	},
	{
	    0,
	    "DEC2030",
	    EISA_PRODUCT_DEC2030,
	},
	{
	    0,
	    "DEC2040",
	    EISA_PRODUCT_DEC2040,
	},
	{
	    0,
	    "DEC2050",
	    EISA_PRODUCT_DEC2050,
	},
	{
	    0,
	    "DEC2400",
	    EISA_PRODUCT_DEC2400,
	},
	{
	    0,
	    "DEC2500",
	    EISA_PRODUCT_DEC2500,
	},
	{
	    0,
	    "DEC2A01",
	    EISA_PRODUCT_DEC2A01,
	},
	{
	    0,
	    "DEC2E00",
	    EISA_PRODUCT_DEC2E00,
	},
	{
	    0,
	    "DEC2F00",
	    EISA_PRODUCT_DEC2F00,
	},
	{
	    0,
	    "DEC3001",
	    EISA_PRODUCT_DEC3001,
	},
	{
	    0,
	    "DEC3002",
	    EISA_PRODUCT_DEC3002,
	},
	{
	    0,
	    "DEC3003",
	    EISA_PRODUCT_DEC3003,
	},
	{
	    0,
	    "DEC3004",
	    EISA_PRODUCT_DEC3004,
	},
	{
	    0,
	    "DEC4220",
	    EISA_PRODUCT_DEC4220,
	},
	{
	    0,
	    "DEC4250",
	    EISA_PRODUCT_DEC4250,
	},
	{
	    0,
	    "DEC5000",
	    EISA_PRODUCT_DEC5000,
	},
	{
	    0,
	    "DEC5100",
	    EISA_PRODUCT_DEC5100,
	},
	{
	    0,
	    "DEC5301",
	    EISA_PRODUCT_DEC5301,
	},
	{
	    0,
	    "DEC6000",
	    EISA_PRODUCT_DEC6000,
	},
	{
	    0,
	    "DEC6400",
	    EISA_PRODUCT_DEC6400,
	},
	{
	    0,
	    "DEC8101",
	    EISA_PRODUCT_DEC8101,
	},
	{
	    0,
	    "DEC8102",
	    EISA_PRODUCT_DEC8102,
	},
	{
	    0,
	    "DEC8103",
	    EISA_PRODUCT_DEC8103,
	},
	{
	    0,
	    "DEC8300",
	    EISA_PRODUCT_DEC8300,
	},
	{
	    0,
	    "DEC8301",
	    EISA_PRODUCT_DEC8301,
	},
	{
	    0,
	    "DEL0000",
	    EISA_PRODUCT_DEL0000,
	},
	{
	    0,
	    "DEL0001",
	    EISA_PRODUCT_DEL0001,
	},
	{
	    0,
	    "DEL0002",
	    EISA_PRODUCT_DEL0002,
	},
	{
	    0,
	    "DEL0003",
	    EISA_PRODUCT_DEL0003,
	},
	{
	    0,
	    "DEL0004",
	    EISA_PRODUCT_DEL0004,
	},
	{
	    0,
	    "DEL0005",
	    EISA_PRODUCT_DEL0005,
	},
	{
	    0,
	    "DEL0006",
	    EISA_PRODUCT_DEL0006,
	},
	{
	    0,
	    "DEL0007",
	    EISA_PRODUCT_DEL0007,
	},
	{
	    0,
	    "DEL0008",
	    EISA_PRODUCT_DEL0008,
	},
	{
	    0,
	    "DEL0009",
	    EISA_PRODUCT_DEL0009,
	},
	{
	    0,
	    "DEL000A",
	    EISA_PRODUCT_DEL000A,
	},
	{
	    0,
	    "DEL000B",
	    EISA_PRODUCT_DEL000B,
	},
	{
	    0,
	    "DEL000C",
	    EISA_PRODUCT_DEL000C,
	},
	{
	    0,
	    "DEL0011",
	    EISA_PRODUCT_DEL0011,
	},
	{
	    0,
	    "DEL0019",
	    EISA_PRODUCT_DEL0019,
	},
	{
	    0,
	    "DEL0021",
	    EISA_PRODUCT_DEL0021,
	},
	{
	    0,
	    "DEL0029",
	    EISA_PRODUCT_DEL0029,
	},
	{
	    0,
	    "DEL002A",
	    EISA_PRODUCT_DEL002A,
	},
	{
	    0,
	    "DEL002B",
	    EISA_PRODUCT_DEL002B,
	},
	{
	    0,
	    "DEL002C",
	    EISA_PRODUCT_DEL002C,
	},
	{
	    0,
	    "DEL002D",
	    EISA_PRODUCT_DEL002D,
	},
	{
	    0,
	    "DEL0031",
	    EISA_PRODUCT_DEL0031,
	},
	{
	    0,
	    "DEL0036",
	    EISA_PRODUCT_DEL0036,
	},
	{
	    0,
	    "DEL0038",
	    EISA_PRODUCT_DEL0038,
	},
	{
	    0,
	    "DEL0054",
	    EISA_PRODUCT_DEL0054,
	},
	{
	    0,
	    "DEL0058",
	    EISA_PRODUCT_DEL0058,
	},
	{
	    0,
	    "DEL005A",
	    EISA_PRODUCT_DEL005A,
	},
	{
	    0,
	    "DEL005C",
	    EISA_PRODUCT_DEL005C,
	},
	{
	    0,
	    "DEL2100",
	    EISA_PRODUCT_DEL2100,
	},
	{
	    0,
	    "DEL4001",
	    EISA_PRODUCT_DEL4001,
	},
	{
	    0,
	    "DEL4002",
	    EISA_PRODUCT_DEL4002,
	},
	{
	    0,
	    "DEL6001",
	    EISA_PRODUCT_DEL6001,
	},
	{
	    0,
	    "DELFC00",
	    EISA_PRODUCT_DELFC00,
	},
	{
	    0,
	    "DELFC01",
	    EISA_PRODUCT_DELFC01,
	},
	{
	    0,
	    "DELFC02",
	    EISA_PRODUCT_DELFC02,
	},
	{
	    0,
	    "DELFC03",
	    EISA_PRODUCT_DELFC03,
	},
	{
	    0,
	    "DELFD00",
	    EISA_PRODUCT_DELFD00,
	},
	{
	    0,
	    "DELFD02",
	    EISA_PRODUCT_DELFD02,
	},
	{
	    0,
	    "DELFD03",
	    EISA_PRODUCT_DELFD03,
	},
	{
	    0,
	    "DELFD05",
	    EISA_PRODUCT_DELFD05,
	},
	{
	    0,
	    "DIS0000",
	    EISA_PRODUCT_DIS0000,
	},
	{
	    0,
	    "DPT2402",
	    EISA_PRODUCT_DPT2402,
	},
	{
	    0,
	    "DPT2403",
	    EISA_PRODUCT_DPT2403,
	},
	{
	    0,
	    "DPTA401",
	    EISA_PRODUCT_DPTA401,
	},
	{
	    0,
	    "DPTA402",
	    EISA_PRODUCT_DPTA402,
	},
	{
	    0,
	    "DPTA410",
	    EISA_PRODUCT_DPTA410,
	},
	{
	    0,
	    "DPTA501",
	    EISA_PRODUCT_DPTA501,
	},
	{
	    0,
	    "DPTA502",
	    EISA_PRODUCT_DPTA502,
	},
	{
	    0,
	    "DPTA701",
	    EISA_PRODUCT_DPTA701,
	},
	{
	    0,
	    "DPTBC01",
	    EISA_PRODUCT_DPTBC01,
	},
	{
	    0,
	    "DTC1101",
	    EISA_PRODUCT_DTC1101,
	},
	{
	    0,
	    "DTC3101",
	    EISA_PRODUCT_DTC3101,
	},
	{
	    0,
	    "DTI0000",
	    EISA_PRODUCT_DTI0000,
	},
	{
	    0,
	    "DTI2000",
	    EISA_PRODUCT_DTI2000,
	},
	{
	    0,
	    "DTI2002",
	    EISA_PRODUCT_DTI2002,
	},
	{
	    0,
	    "DTK0001",
	    EISA_PRODUCT_DTK0001,
	},
	{
	    0,
	    "DTK0003",
	    EISA_PRODUCT_DTK0003,
	},
	{
	    0,
	    "ECS0580",
	    EISA_PRODUCT_ECS0580,
	},
	{
	    0,
	    "ECS0590",
	    EISA_PRODUCT_ECS0590,
	},
	{
	    0,
	    "ELS8041",
	    EISA_PRODUCT_ELS8041,
	},
	{
	    0,
	    "ETI1001",
	    EISA_PRODUCT_ETI1001,
	},
	{
	    0,
	    "EVX0002",
	    EISA_PRODUCT_EVX0002,
	},
	{
	    0,
	    "FCT0001",
	    EISA_PRODUCT_FCT0001,
	},
	{
	    0,
	    "FCT0002",
	    EISA_PRODUCT_FCT0002,
	},
	{
	    0,
	    "FCT0003",
	    EISA_PRODUCT_FCT0003,
	},
	{
	    0,
	    "FIC0000",
	    EISA_PRODUCT_FIC0000,
	},
	{
	    0,
	    "FIX1516",
	    EISA_PRODUCT_FIX1516,
	},
	{
	    0,
	    "FSI2001",
	    EISA_PRODUCT_FSI2001,
	},
	{
	    0,
	    "FSI2002",
	    EISA_PRODUCT_FSI2002,
	},
	{
	    0,
	    "FSI2003",
	    EISA_PRODUCT_FSI2003,
	},
	{
	    0,
	    "GDT2001",
	    EISA_PRODUCT_GDT2001,
	},
	{
	    0,
	    "GDT3001",
	    EISA_PRODUCT_GDT3001,
	},
	{
	    0,
	    "GDT3002",
	    EISA_PRODUCT_GDT3002,
	},
	{
	    0,
	    "GDT3003",
	    EISA_PRODUCT_GDT3003,
	},
	{
	    0,
	    "GIT0000",
	    EISA_PRODUCT_GIT0000,
	},
	{
	    0,
	    "GIT0001",
	    EISA_PRODUCT_GIT0001,
	},
	{
	    0,
	    "HCL0801",
	    EISA_PRODUCT_HCL0801,
	},
	{
	    0,
	    "HIT0001",
	    EISA_PRODUCT_HIT0001,
	},
	{
	    0,
	    "HKG0011",
	    EISA_PRODUCT_HKG0011,
	},
	{
	    0,
	    "HMS0000",
	    EISA_PRODUCT_HMS0000,
	},
	{
	    0,
	    "HWP0000",
	    EISA_PRODUCT_HWP0000,
	},
	{
	    0,
	    "HWP0010",
	    EISA_PRODUCT_HWP0010,
	},
	{
	    0,
	    "HWP0020",
	    EISA_PRODUCT_HWP0020,
	},
	{
	    0,
	    "HWP0030",
	    EISA_PRODUCT_HWP0030,
	},
	{
	    0,
	    "HWP0070",
	    EISA_PRODUCT_HWP0070,
	},
	{
	    0,
	    "HWP0C70",
	    EISA_PRODUCT_HWP0C70,
	},
	{
	    0,
	    "HWP0C80",
	    EISA_PRODUCT_HWP0C80,
	},
	{
	    0,
	    "HWP1400",
	    EISA_PRODUCT_HWP1400,
	},
	{
	    0,
	    "HWP1410",
	    EISA_PRODUCT_HWP1410,
	},
	{
	    0,
	    "HWP1420",
	    EISA_PRODUCT_HWP1420,
	},
	{
	    0,
	    "HWP1440",
	    EISA_PRODUCT_HWP1440,
	},
	{
	    0,
	    "HWP1450",
	    EISA_PRODUCT_HWP1450,
	},
	{
	    0,
	    "HWP1460",
	    EISA_PRODUCT_HWP1460,
	},
	{
	    0,
	    "HWP1461",
	    EISA_PRODUCT_HWP1461,
	},
	{
	    0,
	    "HWP1510",
	    EISA_PRODUCT_HWP1510,
	},
	{
	    0,
	    "HWP1801",
	    EISA_PRODUCT_HWP1801,
	},
	{
	    0,
	    "HWP1810",
	    EISA_PRODUCT_HWP1810,
	},
	{
	    0,
	    "HWP1811",
	    EISA_PRODUCT_HWP1811,
	},
	{
	    0,
	    "HWP1820",
	    EISA_PRODUCT_HWP1820,
	},
	{
	    0,
	    "HWP1832",
	    EISA_PRODUCT_HWP1832,
	},
	{
	    0,
	    "HWP1840",
	    EISA_PRODUCT_HWP1840,
	},
	{
	    0,
	    "HWP1850",
	    EISA_PRODUCT_HWP1850,
	},
	{
	    0,
	    "HWP1870",
	    EISA_PRODUCT_HWP1870,
	},
	{
	    0,
	    "HWP18A0",
	    EISA_PRODUCT_HWP18A0,
	},
	{
	    0,
	    "HWP18C0",
	    EISA_PRODUCT_HWP18C0,
	},
	{
	    0,
	    "HWP18E0",
	    EISA_PRODUCT_HWP18E0,
	},
	{
	    0,
	    "HWP1940",
	    EISA_PRODUCT_HWP1940,
	},
	{
	    0,
	    "HWP1980",
	    EISA_PRODUCT_HWP1980,
	},
	{
	    0,
	    "HWP1990",
	    EISA_PRODUCT_HWP1990,
	},
	{
	    0,
	    "HWP19A0",
	    EISA_PRODUCT_HWP19A0,
	},
	{
	    0,
	    "HWP1C00",
	    EISA_PRODUCT_HWP1C00,
	},
	{
	    0,
	    "HWP2002",
	    EISA_PRODUCT_HWP2002,
	},
	{
	    0,
	    "HWP2051",
	    EISA_PRODUCT_HWP2051,
	},
	{
	    0,
	    "HWP2080",
	    EISA_PRODUCT_HWP2080,
	},
	{
	    0,
	    "HWPC000",
	    EISA_PRODUCT_HWPC000,
	},
	{
	    0,
	    "HWPC010",
	    EISA_PRODUCT_HWPC010,
	},
	{
	    0,
	    "HWPC051",
	    EISA_PRODUCT_HWPC051,
	},
	{
	    0,
	    "HWPC091",
	    EISA_PRODUCT_HWPC091,
	},
	{
	    0,
	    "HWPC0D1",
	    EISA_PRODUCT_HWPC0D1,
	},
	{
	    0,
	    "HWPC0E1",
	    EISA_PRODUCT_HWPC0E1,
	},
	{
	    0,
	    "IBM0001",
	    EISA_PRODUCT_IBM0001,
	},
	{
	    0,
	    "IBM1000",
	    EISA_PRODUCT_IBM1000,
	},
	{
	    0,
	    "IBM1060",
	    EISA_PRODUCT_IBM1060,
	},
	{
	    0,
	    "IBM1061",
	    EISA_PRODUCT_IBM1061,
	},
	{
	    0,
	    "ICLA080",
	    EISA_PRODUCT_ICLA080,
	},
	{
	    0,
	    "IDS0100",
	    EISA_PRODUCT_IDS0100,
	},
	{
	    0,
	    "IIN0B01",
	    EISA_PRODUCT_IIN0B01,
	},
	{
	    0,
	    "IKN1110",
	    EISA_PRODUCT_IKN1110,
	},
	{
	    0,
	    "IMS1001",
	    EISA_PRODUCT_IMS1001,
	},
	{
	    0,
	    "ING2040",
	    EISA_PRODUCT_ING2040,
	},
	{
	    0,
	    "INP0010",
	    EISA_PRODUCT_INP0010,
	},
	{
	    0,
	    "INP25D0",
	    EISA_PRODUCT_INP25D0,
	},
	{
	    0,
	    "INP5000",
	    EISA_PRODUCT_INP5000,
	},
	{
	    0,
	    "INT0000",
	    EISA_PRODUCT_INT0000,
	},
	{
	    0,
	    "INT0081",
	    EISA_PRODUCT_INT0081,
	},
	{
	    0,
	    "INT0701",
	    EISA_PRODUCT_INT0701,
	},
	{
	    0,
	    "INT0703",
	    EISA_PRODUCT_INT0703,
	},
	{
	    0,
	    "INT0902",
	    EISA_PRODUCT_INT0902,
	},
	{
	    0,
	    "INT0B01",
	    EISA_PRODUCT_INT0B01,
	},
	{
	    0,
	    "INT1000",
	    EISA_PRODUCT_INT1000,
	},
	{
	    0,
	    "INT1010",
	    EISA_PRODUCT_INT1010,
	},
	{
	    0,
	    "INT1030",
	    EISA_PRODUCT_INT1030,
	},
	{
	    0,
	    "INT1031",
	    EISA_PRODUCT_INT1031,
	},
	{
	    0,
	    "INT1060",
	    EISA_PRODUCT_INT1060,
	},
	{
	    0,
	    "INT1201",
	    EISA_PRODUCT_INT1201,
	},
	{
	    0,
	    "INT3061",
	    EISA_PRODUCT_INT3061,
	},
	{
	    0,
	    "INT3079",
	    EISA_PRODUCT_INT3079,
	},
	{
	    0,
	    "INT3089",
	    EISA_PRODUCT_INT3089,
	},
	{
	    0,
	    "INT30A1",
	    EISA_PRODUCT_INT30A1,
	},
	{
	    0,
	    "INT30A9",
	    EISA_PRODUCT_INT30A9,
	},
	{
	    0,
	    "INT30D1",
	    EISA_PRODUCT_INT30D1,
	},
	{
	    0,
	    "INT30F1",
	    EISA_PRODUCT_INT30F1,
	},
	{
	    0,
	    "INT3132",
	    EISA_PRODUCT_INT3132,
	},
	{
	    0,
	    "INT31A0",
	    EISA_PRODUCT_INT31A0,
	},
	{
	    0,
	    "ISY0010",
	    EISA_PRODUCT_ISY0010,
	},
	{
	    0,
	    "ISY0020",
	    EISA_PRODUCT_ISY0020,
	},
	{
	    0,
	    "ISY0030",
	    EISA_PRODUCT_ISY0030,
	},
	{
	    0,
	    "ISY0040",
	    EISA_PRODUCT_ISY0040,
	},
	{
	    0,
	    "ISY0050",
	    EISA_PRODUCT_ISY0050,
	},
	{
	    0,
	    "ISY0060",
	    EISA_PRODUCT_ISY0060,
	},
	{
	    0,
	    "ISY0070",
	    EISA_PRODUCT_ISY0070,
	},
	{
	    0,
	    "ITC0001",
	    EISA_PRODUCT_ITC0001,
	},
	{
	    0,
	    "ITK0011",
	    EISA_PRODUCT_ITK0011,
	},
	{
	    0,
	    "ITK0012",
	    EISA_PRODUCT_ITK0012,
	},
	{
	    0,
	    "KCI3201",
	    EISA_PRODUCT_KCI3201,
	},
	{
	    0,
	    "KCI3202",
	    EISA_PRODUCT_KCI3202,
	},
	{
	    0,
	    "LEF8000",
	    EISA_PRODUCT_LEF8000,
	},
	{
	    0,
	    "MDG0002",
	    EISA_PRODUCT_MDG0002,
	},
	{
	    0,
	    "MDG0010",
	    EISA_PRODUCT_MDG0010,
	},
	{
	    0,
	    "MDG0020",
	    EISA_PRODUCT_MDG0020,
	},
	{
	    0,
	    "MDG2000",
	    EISA_PRODUCT_MDG2000,
	},
	{
	    0,
	    "MDG2010",
	    EISA_PRODUCT_MDG2010,
	},
	{
	    0,
	    "MCC0000",
	    EISA_PRODUCT_MCC0000,
	},
	{
	    0,
	    "MCC0001",
	    EISA_PRODUCT_MCC0001,
	},
	{
	    0,
	    "MCY2501",
	    EISA_PRODUCT_MCY2501,
	},
	{
	    0,
	    "MET1104",
	    EISA_PRODUCT_MET1104,
	},
	{
	    0,
	    "MET1128",
	    EISA_PRODUCT_MET1128,
	},
	{
	    0,
	    "MIC0001",
	    EISA_PRODUCT_MIC0001,
	},
	{
	    0,
	    "MIC0004",
	    EISA_PRODUCT_MIC0004,
	},
	{
	    0,
	    "MIC0005",
	    EISA_PRODUCT_MIC0005,
	},
	{
	    0,
	    "MIC0021",
	    EISA_PRODUCT_MIC0021,
	},
	{
	    0,
	    "MIC0054",
	    EISA_PRODUCT_MIC0054,
	},
	{
	    0,
	    "MIC3001",
	    EISA_PRODUCT_MIC3001,
	},
	{
	    0,
	    "MIC5402",
	    EISA_PRODUCT_MIC5402,
	},
	{
	    0,
	    "MIR0928",
	    EISA_PRODUCT_MIR0928,
	},
	{
	    0,
	    "MLX0010",
	    EISA_PRODUCT_MLX0010,
	},
	{
	    0,
	    "MLX0011",
	    EISA_PRODUCT_MLX0011,
	},
	{
	    0,
	    "MLX0020",
	    EISA_PRODUCT_MLX0020,
	},
	{
	    0,
	    "MLX0021",
	    EISA_PRODUCT_MLX0021,
	},
	{
	    0,
	    "MLX0022",
	    EISA_PRODUCT_MLX0022,
	},
	{
	    0,
	    "MLX0030",
	    EISA_PRODUCT_MLX0030,
	},
	{
	    0,
	    "MLX0040",
	    EISA_PRODUCT_MLX0040,
	},
	{
	    0,
	    "MLX0050",
	    EISA_PRODUCT_MLX0050,
	},
	{
	    0,
	    "MLX0070",
	    EISA_PRODUCT_MLX0070,
	},
	{
	    0,
	    "MLX0071",
	    EISA_PRODUCT_MLX0071,
	},
	{
	    0,
	    "MLX0072",
	    EISA_PRODUCT_MLX0072,
	},
	{
	    0,
	    "MLX0073",
	    EISA_PRODUCT_MLX0073,
	},
	{
	    0,
	    "MLX0074",
	    EISA_PRODUCT_MLX0074,
	},
	{
	    0,
	    "MLX0075",
	    EISA_PRODUCT_MLX0075,
	},
	{
	    0,
	    "MLX0076",
	    EISA_PRODUCT_MLX0076,
	},
	{
	    0,
	    "MLX0077",
	    EISA_PRODUCT_MLX0077,
	},
	{
	    0,
	    "MLX0101",
	    EISA_PRODUCT_MLX0101,
	},
	{
	    0,
	    "MLXFD01",
	    EISA_PRODUCT_MLXFD01,
	},
	{
	    0,
	    "MLXFE01",
	    EISA_PRODUCT_MLXFE01,
	},
	{
	    0,
	    "MLXFF01",
	    EISA_PRODUCT_MLXFF01,
	},
	{
	    0,
	    "MLXFF02",
	    EISA_PRODUCT_MLXFF02,
	},
	{
	    0,
	    "MTX2040",
	    EISA_PRODUCT_MTX2040,
	},
	{
	    0,
	    "NEC8201",
	    EISA_PRODUCT_NEC8201,
	},
	{
	    0,
	    "NIC0202",
	    EISA_PRODUCT_NIC0202,
	},
	{
	    0,
	    "NIC0301",
	    EISA_PRODUCT_NIC0301,
	},
	{
	    0,
	    "NIC0400",
	    EISA_PRODUCT_NIC0400,
	},
	{
	    0,
	    "NIC0501",
	    EISA_PRODUCT_NIC0501,
	},
	{
	    0,
	    "NIC0602",
	    EISA_PRODUCT_NIC0602,
	},
	{
	    0,
	    "NIC0700",
	    EISA_PRODUCT_NIC0700,
	},
	{
	    0,
	    "NIC0800",
	    EISA_PRODUCT_NIC0800,
	},
	{
	    0,
	    "NIC0900",
	    EISA_PRODUCT_NIC0900,
	},
	{
	    0,
	    "NIC1000",
	    EISA_PRODUCT_NIC1000,
	},
	{
	    0,
	    "NIC1100",
	    EISA_PRODUCT_NIC1100,
	},
	{
	    0,
	    "NIC1200",
	    EISA_PRODUCT_NIC1200,
	},
	{
	    0,
	    "NIC1300",
	    EISA_PRODUCT_NIC1300,
	},
	{
	    0,
	    "NIC1400",
	    EISA_PRODUCT_NIC1400,
	},
	{
	    0,
	    "NIC1500",
	    EISA_PRODUCT_NIC1500,
	},
	{
	    0,
	    "NICC005",
	    EISA_PRODUCT_NICC005,
	},
	{
	    0,
	    "NICC105",
	    EISA_PRODUCT_NICC105,
	},
	{
	    0,
	    "NICC205",
	    EISA_PRODUCT_NICC205,
	},
	{
	    0,
	    "NICC304",
	    EISA_PRODUCT_NICC304,
	},
	{
	    0,
	    "NICC502",
	    EISA_PRODUCT_NICC502,
	},
	{
	    0,
	    "NON0101",
	    EISA_PRODUCT_NON0101,
	},
	{
	    0,
	    "NON0102",
	    EISA_PRODUCT_NON0102,
	},
	{
	    0,
	    "NON0201",
	    EISA_PRODUCT_NON0201,
	},
	{
	    0,
	    "NON0301",
	    EISA_PRODUCT_NON0301,
	},
	{
	    0,
	    "NON0401",
	    EISA_PRODUCT_NON0401,
	},
	{
	    0,
	    "NON0501",
	    EISA_PRODUCT_NON0501,
	},
	{
	    0,
	    "NON0601",
	    EISA_PRODUCT_NON0601,
	},
	{
	    0,
	    "NSS0011",
	    EISA_PRODUCT_NSS0011,
	},
	{
	    0,
	    "NVL0701",
	    EISA_PRODUCT_NVL0701,
	},
	{
	    0,
	    "NVL0901",
	    EISA_PRODUCT_NVL0901,
	},
	{
	    0,
	    "NVL1001",
	    EISA_PRODUCT_NVL1001,
	},
	{
	    0,
	    "NVL1201",
	    EISA_PRODUCT_NVL1201,
	},
	{
	    0,
	    "NVL1301",
	    EISA_PRODUCT_NVL1301,
	},
	{
	    0,
	    "NVL1401",
	    EISA_PRODUCT_NVL1401,
	},
	{
	    0,
	    "NVL1501",
	    EISA_PRODUCT_NVL1501,
	},
	{
	    0,
	    "OLC0701",
	    EISA_PRODUCT_OLC0701,
	},
	{
	    0,
	    "OLC0702",
	    EISA_PRODUCT_OLC0702,
	},
	{
	    0,
	    "OLC0801",
	    EISA_PRODUCT_OLC0801,
	},
	{
	    0,
	    "OLC0901",
	    EISA_PRODUCT_OLC0901,
	},
	{
	    0,
	    "OLC0902",
	    EISA_PRODUCT_OLC0902,
	},
	{
	    0,
	    "OLC0B01",
	    EISA_PRODUCT_OLC0B01,
	},
	{
	    0,
	    "OLC1201",
	    EISA_PRODUCT_OLC1201,
	},
	{
	    0,
	    "OPT0000",
	    EISA_PRODUCT_OPT0000,
	},
	{
	    0,
	    "OPT0200",
	    EISA_PRODUCT_OPT0200,
	},
	{
	    0,
	    "OTI0011",
	    EISA_PRODUCT_OTI0011,
	},
	{
	    0,
	    "PCI0080",
	    EISA_PRODUCT_PCI0080,
	},
	{
	    0,
	    "PCI0120",
	    EISA_PRODUCT_PCI0120,
	},
	{
	    0,
	    "PCI2080",
	    EISA_PRODUCT_PCI2080,
	},
	{
	    0,
	    "PHI8041",
	    EISA_PRODUCT_PHI8041,
	},
	{
	    0,
	    "PLX1001",
	    EISA_PRODUCT_PLX1001,
	},
	{
	    0,
	    "PRO6000",
	    EISA_PRODUCT_PRO6000,
	},
	{
	    0,
	    "PRO6001",
	    EISA_PRODUCT_PRO6001,
	},
	{
	    0,
	    "PRO6002",
	    EISA_PRODUCT_PRO6002,
	},
	{
	    0,
	    "PTI5401",
	    EISA_PRODUCT_PTI5401,
	},
	{
	    0,
	    "RII0101",
	    EISA_PRODUCT_RII0101,
	},
	{
	    0,
	    "SEC0010",
	    EISA_PRODUCT_SEC0010,
	},
	{
	    0,
	    "SEC0020",
	    EISA_PRODUCT_SEC0020,
	},
	{
	    0,
	    "SEC0021",
	    EISA_PRODUCT_SEC0021,
	},
	{
	    0,
	    "SECFF01",
	    EISA_PRODUCT_SECFF01,
	},
	{
	    0,
	    "SECFF02",
	    EISA_PRODUCT_SECFF02,
	},
	{
	    0,
	    "SGT0101",
	    EISA_PRODUCT_SGT0101,
	},
	{
	    0,
	    "SIS0000",
	    EISA_PRODUCT_SIS0000,
	},
	{
	    0,
	    "SIS0001",
	    EISA_PRODUCT_SIS0001,
	},
	{
	    0,
	    "SKD0100",
	    EISA_PRODUCT_SKD0100,
	},
	{
	    0,
	    "SMC0110",
	    EISA_PRODUCT_SMC0110,
	},
	{
	    0,
	    "SMC03E0",
	    EISA_PRODUCT_SMC03E0,
	},
	{
	    0,
	    "SMC13E0",
	    EISA_PRODUCT_SMC13E0,
	},
	{
	    0,
	    "SMC8003",
	    EISA_PRODUCT_SMC8003,
	},
	{
	    0,
	    "SMC8010",
	    EISA_PRODUCT_SMC8010,
	},
	{
	    0,
	    "SMC8013",
	    EISA_PRODUCT_SMC8013,
	},
	{
	    0,
	    "SMC8216",
	    EISA_PRODUCT_SMC8216,
	},
	{
	    0,
	    "SMCA010",
	    EISA_PRODUCT_SMCA010,
	},
	{
	    0,
	    "SNIAAA0",
	    EISA_PRODUCT_SNIAAA0,
	},
	{
	    0,
	    "SNIAAB0",
	    EISA_PRODUCT_SNIAAB0,
	},
	{
	    0,
	    "SNIABF0",
	    EISA_PRODUCT_SNIABF0,
	},
	{
	    0,
	    "SNIACA0",
	    EISA_PRODUCT_SNIACA0,
	},
	{
	    0,
	    "STL0100",
	    EISA_PRODUCT_STL0100,
	},
	{
	    0,
	    "STL0120",
	    EISA_PRODUCT_STL0120,
	},
	{
	    0,
	    "STL0130",
	    EISA_PRODUCT_STL0130,
	},
	{
	    0,
	    "STL0200",
	    EISA_PRODUCT_STL0200,
	},
	{
	    0,
	    "STL0400",
	    EISA_PRODUCT_STL0400,
	},
	{
	    0,
	    "STL0410",
	    EISA_PRODUCT_STL0410,
	},
	{
	    0,
	    "STL0420",
	    EISA_PRODUCT_STL0420,
	},
	{
	    0,
	    "SUK1022",
	    EISA_PRODUCT_SUK1022,
	},
	{
	    0,
	    "SUK1059",
	    EISA_PRODUCT_SUK1059,
	},
	{
	    0,
	    "SUK1072",
	    EISA_PRODUCT_SUK1072,
	},
	{
	    0,
	    "TCC010C",
	    EISA_PRODUCT_TCC010C,
	},
	{
	    0,
	    "TCC030D",
	    EISA_PRODUCT_TCC030D,
	},
	{
	    0,
	    "TCC040B",
	    EISA_PRODUCT_TCC040B,
	},
	{
	    0,
	    "TCC3047",
	    EISA_PRODUCT_TCC3047,
	},
	{
	    0,
	    "TCM3190",
	    EISA_PRODUCT_TCM3190,
	},
	{
	    0,
	    "TCM5030",
	    EISA_PRODUCT_TCM5030,
	},
	{
	    0,
	    "TCM5070",
	    EISA_PRODUCT_TCM5070,
	},
	{
	    0,
	    "TCM5090",
	    EISA_PRODUCT_TCM5090,
	},
	{
	    0,
	    "TCM5091",
	    EISA_PRODUCT_TCM5091,
	},
	{
	    0,
	    "TCM5092",
	    EISA_PRODUCT_TCM5092,
	},
	{
	    0,
	    "TCM5093",
	    EISA_PRODUCT_TCM5093,
	},
	{
	    0,
	    "TCM5094",
	    EISA_PRODUCT_TCM5094,
	},
	{
	    0,
	    "TCM5095",
	    EISA_PRODUCT_TCM5095,
	},
	{
	    0,
	    "TCM5098",
	    EISA_PRODUCT_TCM5098,
	},
	{
	    0,
	    "TCM5920",
	    EISA_PRODUCT_TCM5920,
	},
	{
	    0,
	    "TCM5970",
	    EISA_PRODUCT_TCM5970,
	},
	{
	    0,
	    "TCM5971",
	    EISA_PRODUCT_TCM5971,
	},
	{
	    0,
	    "TCM5972",
	    EISA_PRODUCT_TCM5972,
	},
	{
	    0,
	    "TCM7700",
	    EISA_PRODUCT_TCM7700,
	},
	{
	    0,
	    "TCO010C",
	    EISA_PRODUCT_TCO010C,
	},
	{
	    0,
	    "TCO030D",
	    EISA_PRODUCT_TCO030D,
	},
	{
	    0,
	    "TCO040B",
	    EISA_PRODUCT_TCO040B,
	},
	{
	    0,
	    "TCO050D",
	    EISA_PRODUCT_TCO050D,
	},
	{
	    0,
	    "TCO3147",
	    EISA_PRODUCT_TCO3147,
	},
	{
	    0,
	    "TCO345A",
	    EISA_PRODUCT_TCO345A,
	},
	{
	    0,
	    "TCO345B",
	    EISA_PRODUCT_TCO345B,
	},
	{
	    0,
	    "TEC8000",
	    EISA_PRODUCT_TEC8000,
	},
	{
	    0,
	    "TEC8001",
	    EISA_PRODUCT_TEC8001,
	},
	{
	    0,
	    "TEC8002",
	    EISA_PRODUCT_TEC8002,
	},
	{
	    0,
	    "TEC8003",
	    EISA_PRODUCT_TEC8003,
	},
	{
	    0,
	    "TRM0001",
	    EISA_PRODUCT_TRM0001,
	},
	{
	    0,
	    "TRM0320",
	    EISA_PRODUCT_TRM0320,
	},
	{
	    0,
	    "TRM0620",
	    EISA_PRODUCT_TRM0620,
	},
	{
	    0,
	    "TRM0820",
	    EISA_PRODUCT_TRM0820,
	},
	{
	    0,
	    "TRM320E",
	    EISA_PRODUCT_TRM320E,
	},
	{
	    0,
	    "TRM820B",
	    EISA_PRODUCT_TRM820B,
	},
	{
	    0,
	    "TRU0210",
	    EISA_PRODUCT_TRU0210,
	},
	{
	    0,
	    "TRU0520",
	    EISA_PRODUCT_TRU0520,
	},
	{
	    0,
	    "TRU1100",
	    EISA_PRODUCT_TRU1100,
	},
	{
	    0,
	    "TXN0011",
	    EISA_PRODUCT_TXN0011,
	},
	{
	    0,
	    "TYN0000",
	    EISA_PRODUCT_TYN0000,
	},
	{
	    0,
	    "TYN0001",
	    EISA_PRODUCT_TYN0001,
	},
	{
	    0,
	    "TYN0003",
	    EISA_PRODUCT_TYN0003,
	},
	{
	    0,
	    "UBIA100",
	    EISA_PRODUCT_UBIA100,
	},
	{
	    0,
	    "UBIA200",
	    EISA_PRODUCT_UBIA200,
	},
	{
	    0,
	    "UBIB100",
	    EISA_PRODUCT_UBIB100,
	},
	{
	    0,
	    "UBIB200",
	    EISA_PRODUCT_UBIB200,
	},
	{
	    0,
	    "UBIC100",
	    EISA_PRODUCT_UBIC100,
	},
	{
	    0,
	    "UBID100",
	    EISA_PRODUCT_UBID100,
	},
	{
	    0,
	    "USC0120",
	    EISA_PRODUCT_USC0120,
	},
	{
	    0,
	    "USC0125",
	    EISA_PRODUCT_USC0125,
	},
	{
	    0,
	    "USC0140",
	    EISA_PRODUCT_USC0140,
	},
	{
	    0,
	    "USC0220",
	    EISA_PRODUCT_USC0220,
	},
	{
	    0,
	    "USC0225",
	    EISA_PRODUCT_USC0225,
	},
	{
	    0,
	    "USC0240",
	    EISA_PRODUCT_USC0240,
	},
	{
	    0,
	    "USC0340",
	    EISA_PRODUCT_USC0340,
	},
	{
	    0,
	    "USR0011",
	    EISA_PRODUCT_USR0011,
	},
	{
	    0,
	    "USR3401",
	    EISA_PRODUCT_USR3401,
	},
	{
	    0,
	    "VMI0201",
	    EISA_PRODUCT_VMI0201,
	},
	{
	    0,
	    "VMI0211",
	    EISA_PRODUCT_VMI0211,
	},
	{
	    0,
	    "VMI0601",
	    EISA_PRODUCT_VMI0601,
	},
	{
	    0,
	    "VMI0E01",
	    EISA_PRODUCT_VMI0E01,
	},
	{
	    0,
	    "WDC0101",
	    EISA_PRODUCT_WDC0101,
	},
	{
	    0,
	    "WDC0102",
	    EISA_PRODUCT_WDC0102,
	},
	{
	    0,
	    "WDC0300",
	    EISA_PRODUCT_WDC0300,
	},
	{
	    0,
	    "WDC0301",
	    EISA_PRODUCT_WDC0301,
	},
	{
	    0,
	    "WDC03E0",
	    EISA_PRODUCT_WDC03E0,
	},
	{
	    0,
	    "WDC03E1",
	    EISA_PRODUCT_WDC03E1,
	},
	{
	    0,
	    "WDC03E2",
	    EISA_PRODUCT_WDC03E2,
	},
	{
	    0,
	    "WDC03E3",
	    EISA_PRODUCT_WDC03E3,
	},
	{
	    0,
	    "WDC03E4",
	    EISA_PRODUCT_WDC03E4,
	},
	{
	    0,
	    "WDC0510",
	    EISA_PRODUCT_WDC0510,
	},
	{
	    0,
	    "WDC1009",
	    EISA_PRODUCT_WDC1009,
	},
	{
	    0,
	    "WDC13E0",
	    EISA_PRODUCT_WDC13E0,
	},
	{
	    0,
	    "XTI02B1",
	    EISA_PRODUCT_XTI02B1,
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "ABP",
	    "Advansys",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "ACC",
	    "Accton",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "ACE",
	    "ACME",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "ACR",
	    "Acer",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "ADI",
	    "Lightning Networks",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "ADP",
	    "Adaptec",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "AIR",
	    "AIR",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "AMI",
	    "AMI",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "ASU",
	    "Asus",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "ATI",
	    "ATI",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "BAN",
	    "Banyan Vines",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "BUS",
	    "BusLogic",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "CCI",
	    "Cache Computers, Inc.",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "CHA",
	    "Chase",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "CMD",
	    "CMD Technology, Inc.",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "COG",
	    "Cogent",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "CPQ",
	    "Compaq",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "DBI",
	    "Digi",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "DEC",
	    "Digital Equipment",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "DEL",
	    "Dell",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "DPT",
	    "DPT",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "DTI",
	    "DeskStation Technology, Inc.",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "DTK",
	    "DTK",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "ELS",
	    "ELSA",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "ETI",
	    "ETI",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "FSI",
	    "FORE Systems",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "GDT",
	    "GDT",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "HWP",
	    "Hewlett-Packard",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "IKN",
	    "IKON",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "IMS",
	    "Integrated Micro Solution Inc.",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "INP",
	    "Interphase",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "INT",
	    "Intel",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "ISY",
	    "(SYSTEM)",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "LEF",
	    "LeafNET",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "MCY",
	    "Microdyne",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "MET",
	    "Metheus",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "MIC",
	    "Micronics",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "MLX",
	    "Mylex",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "MDG",
	    "Madge Networks",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "MTX",
	    "Matrox",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "NEC",
	    "NEC",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "NIC",
	    "National Instruments Corp.",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "NON",
	    "c't Universal",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "NSS",
	    "Newport Systems Solutions",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "NVL",
	    "Novell",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "OLC",
	    "Olicom",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "OPT",
	    "OPTi",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "PCI",
	    "Pioneer",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "PRO",
	    "Proteon",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "PTI",
	    "Poseidon",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "SMC",
	    "Standard Microsystems Corp.",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "SNI",
	    "Siemens Nixdorf",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "STL",
	    "Stallion Technologies",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "TCC",
	    "Thomas-Conrad",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "TCM",
	    "3Com",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "TCO",
	    "Thomas-Conrad",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "TEC",
	    "Tecmar",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "TRU",
	    "Truevision",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "TXN",
	    "Texas Instruments",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "TYN",
	    "Tyan",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "UBI",
	    "Ungermann-Bass",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "USC",
	    "UltraStor",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "USR",
	    "US Robotics",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "VMI",
	    "Vermont",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "WDC",
	    "Western Digital",
	},
	{
	    EISA_KNOWNDEV_NOPROD,
	    "XTI",
	    "XTI",
	},
	{ 0, "", NULL, }
};
