/*
 * THIS FILE AUTOMATICALLY GENERATED.  DO NOT EDIT.
 *
 * generated from:
 *	OpenBSD: eisadevs,v 1.20 2004/05/05 05:33:56 mickey Exp 
 */

/*
 * Copyright (c) 1995, 1996 Christopher G. Demetriou
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Christopher G. Demetriou
 *	for the NetBSD Project.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * List of known products, grouped by vendor.
 */

/* Advansys products */
#define	EISA_PRODUCT_ABP0510	"Advansys ABP-510 SCSI"
#define	EISA_PRODUCT_ABP0540	"Advansys ABP-540/542 SCSI"
#define	EISA_PRODUCT_ABP7401	"Advansys ABP-740/742 SCSI"
#define	EISA_PRODUCT_ABP7501	"Advansys ABP-750/752 SCSI"

/* Accton products */
#define	EISA_PRODUCT_ACC1200	"Accton EtherCombo-32 Ethernet"
#define	EISA_PRODUCT_ACC120A	"Accton EtherCombo-32 Ethernet"
#define	EISA_PRODUCT_ACC1650	"Accton EN165x Ethernet"
#define	EISA_PRODUCT_ACC1660	"Accton EN166x Ethernet"

/* ACME products */
#define	EISA_PRODUCT_ACE1010	"ACME Super Fast System Board"
#define	EISA_PRODUCT_ACE2010	"ACME PC Network"
#define	EISA_PRODUCT_ACE3010	"ACME Arcnet Plan"
#define	EISA_PRODUCT_ACE3030	"ACME Sample VS Board 1"
#define	EISA_PRODUCT_ACE4010	"ACME Tape Controller"
#define	EISA_PRODUCT_ACE5010	"ACME VDU Video Board"
#define	EISA_PRODUCT_ACE6010	"ACME Disk Controller"
#define	EISA_PRODUCT_ACE7010	"ACME Multi-Function Board"

/* Acer products */
#define	EISA_PRODUCT_ACR1201	"Acer 1200 486/25 System Board"
#define	EISA_PRODUCT_ACR1211	"Acer AcerFrame 3000SP33 486/33 System Board"
#define	EISA_PRODUCT_ACR1341	"Acer M1 486SX/20 CPU Board"
#define	EISA_PRODUCT_ACR1351	"Acer M1 486SX/20 CPU Board"
#define	EISA_PRODUCT_ACR1361	"Acer M1 487/20 CPU Board"
#define	EISA_PRODUCT_ACR1371	"Acer M1 487/20 CPU Board"
#define	EISA_PRODUCT_ACR1381	"Acer M1 486/20 CPU Board"
#define	EISA_PRODUCT_ACR1391	"Acer M1 486/20 CPU Board"
#define	EISA_PRODUCT_ACR1581	"Acer M1 486/33 CPU Board"
#define	EISA_PRODUCT_ACR1591	"Acer M1 486/33 CPU Board"
#define	EISA_PRODUCT_ACR15A1	"Acer M1 486/33 CPU Board"
#define	EISA_PRODUCT_ACR15B1	"Acer M1 486/33 CPU Board"
#define	EISA_PRODUCT_ACR1701	"Acer AcerFrame 1000"
#define	EISA_PRODUCT_ACR1711	"Acer AcerFrame 1000 486/33 SYSTEM-2"
#define	EISA_PRODUCT_ACR1801	"Acer P43WE System Board"
#define	EISA_PRODUCT_ACR3211	"Acer AcerFrame 3000MP 486 SYSTEM-1"
#define	EISA_PRODUCT_ACR3221	"Acer AcerFrame 486 Series SYSTEM-2"
#define	EISA_PRODUCT_ACR3231	"Acer AcerFrame 486 Series SYSTEM-3"
#define	EISA_PRODUCT_ACR3241	"Acer AcerFrame 486 Series SYSTEM-4"
#define	EISA_PRODUCT_ACR3261	"Acer AcerFrame 3000MP 486 SYSTEM-1"
#define	EISA_PRODUCT_ACR3271	"Acer AcerFrame 486 Series SYSTEM-2"
#define	EISA_PRODUCT_ACR3281	"Acer AcerFrame 486 Series SYSTEM-3"
#define	EISA_PRODUCT_ACR3291	"Acer AcerFrame 486 Series SYSTEM-4"
#define	EISA_PRODUCT_ACR4509	"Acer Altos M1 System Board"

#define	EISA_PRODUCT_ADI0001	"Lightning Networks 32-Bit Ethernet"

/* Adaptec products */
#define	EISA_PRODUCT_ADP0000	"Adaptec AHA-1740 SCSI"
#define	EISA_PRODUCT_ADP0001	"Adaptec AHA-1740A SCSI"
#define	EISA_PRODUCT_ADP0002	"Adaptec AHA-1742A SCSI"
#define	EISA_PRODUCT_ADP0100	"Adaptec AHA-1540/1542 ISA SCSI"
#define	EISA_PRODUCT_ADP0200	"Adaptec AHA-1520/1522 ISA SCSI"
#define	EISA_PRODUCT_ADP0400	"Adaptec AHA-1744 SCSI"
#define	EISA_PRODUCT_ADP7770	"Adaptec AIC-7770 SCSI (on motherboard)"
#define	EISA_PRODUCT_ADP7771	"Adaptec AHA-274x SCSI"
#define	EISA_PRODUCT_ADP7756	"Adaptec AHA-284x SCSI (BIOS enabled)"
#define	EISA_PRODUCT_ADP7757	"Adaptec AHA-284x SCSI (BIOS disabled)"

#define	EISA_PRODUCT_AEI0401	"486EI System Board"

#define	EISA_PRODUCT_AEO0301	"486EO System Board"

#define	EISA_PRODUCT_AIM0002	"AUVA OPTi/EISA 32-Bit 486 All-in-One System Board"

#define	EISA_PRODUCT_AIR0101	"AIR AIR486SE/25/33 Baby AT-foot print motherboard"
#define	EISA_PRODUCT_AIR0103	"AIR AIR486SE/25/33/50"
#define	EISA_PRODUCT_AIR0201	"AIR AIR486LE/25/33/50"
#define	EISA_PRODUCT_AIR0301	"AIR AIR 486EO System Board"
#define	EISA_PRODUCT_AIR0401	"AIR 486EI System Board"
#define	EISA_PRODUCT_AIR0501	"AIR AIR 586EP PCI/EISA System Board"
#define	EISA_PRODUCT_AIR0601	"AIR AIR 54CEP PCI/EISA System Board"
#define	EISA_PRODUCT_AIR0701	"AIR AIR 54CDP PCI/EISA System Board"
#define	EISA_PRODUCT_AIR0702	"AIR AIR 54CDP PCI/EISA Dual-Processors System Board"
#define	EISA_PRODUCT_AIR0901	"AIR AIR 54TDP PCI/EISA Dual-Processors System Board"
#define	EISA_PRODUCT_AIR1001	"AIR AIR P6NDP PCI/EISA Dual-Pentium Processor System Board"
#define	EISA_PRODUCT_AIR2001	"AIR AIR SCSI-2E"
#define	EISA_PRODUCT_AIR2101	"AIR AIR SCSI-2V"
#define	EISA_PRODUCT_AIR3001	"AIR ENET-2E BUS MASTER ETHERNET"
#define	EISA_PRODUCT_AIR3101	"AIR ENET-2V LOCAL BUS MASTER ETHERNET"

#define	EISA_PRODUCT_ALR0001	"Power/Business VEISA System Board"
#define	EISA_PRODUCT_ALR0041	"PowerPro System Board"
#define	EISA_PRODUCT_ALR0181	"PowerPro System Board"
#define	EISA_PRODUCT_ALR0241	"Evolution V Pentium Tower System Board"
#define	EISA_PRODUCT_ALR0341	"EISA/PCI base System Board"
#define	EISA_PRODUCT_ALR3000	"80486 Processor Module"
#define	EISA_PRODUCT_ALR3010	"Pentium Processor Board"
#define	EISA_PRODUCT_ALR3023	"ALR 16-bit VGA without Parallel port"
#define	EISA_PRODUCT_ALR8580	"Advanced Disk Array Caching Controller"
#define	EISA_PRODUCT_ALRA0C1	"System Board"
#define	EISA_PRODUCT_ALRA301	"Revolution Q-SMP System Board"
#define	EISA_PRODUCT_ALRA311	"Revolution Q-2SMP System Board"
#define	EISA_PRODUCT_ALRB0A0	"Primary System Processor Board - 80486DX2/66"
#define	EISA_PRODUCT_ALRB0B0	"Secondary System Processor Board - 80486DX2/66"

/* AMI products */
#define	EISA_PRODUCT_AMI15C1	"AMI SCSI Host Adapter"
#define	EISA_PRODUCT_AMI15D1	"AMI SCSI Host Adapter - Rev 2"
#define	EISA_PRODUCT_AMI15E1	"AMI Single Ended SCSI"
#define	EISA_PRODUCT_AMI16B1	"AMI ENTERPRISE System Board"
#define	EISA_PRODUCT_AMI2509	"AMI ENTERPRISE System Board"
#define	EISA_PRODUCT_AMI25B1	"AMI ENTERPRISE System Board"
#define	EISA_PRODUCT_AMI28A1	"AMI EZ-FLEX System Board"
#define	EISA_PRODUCT_AMI44C1	"AMI SCSI Host Adapter - Series 44"
#define	EISA_PRODUCT_AMI44D2	"AMI Fast Single Ended SCSI"
#define	EISA_PRODUCT_AMI4801	"AMI Series 48 SCSI"
#define	EISA_PRODUCT_AMI68B1	"AMI Enterprise III 486 System Board"

#define	EISA_PRODUCT_APS0101	"PIP INTERFACE"
#define	EISA_PRODUCT_APS0102	"PIP INTERFACE"
#define	EISA_PRODUCT_APS0103	"PIP INTERFACE"

#define	EISA_PRODUCT_ARC0010	"Alta EtherTPI/Combo"
#define	EISA_PRODUCT_ARC0020	"Alta TokenCombo-16 S/U"

#define	EISA_PRODUCT_ASU0100	"Asus SC100 SCSI Cache Host Adapter (CFG file V2.0)"
#define	EISA_PRODUCT_ASU0500	"Asus L500 Ethernet"
#define	EISA_PRODUCT_ASU4001	"Asus 486C Main Board"
#define	EISA_PRODUCT_ASU4101	"Asus 486E Main Board"
#define	EISA_PRODUCT_ASU4201	"Asus 486A Main Board"
#define	EISA_PRODUCT_ASU4301	"Asus 486SI Main Board"
#define	EISA_PRODUCT_ASU4501	"Asus Mini 486H Main Board"
#define	EISA_PRODUCT_ASU4701	"Asus Mini 486AS Main Board"
#define	EISA_PRODUCT_ASU4901	"Asus VL/EISA-486SV1 Main Board"
#define	EISA_PRODUCT_ASU5101	"Asus PCI/E-P5MP4 or PCI/E-P54NP4 Main Board V2.3"
#define	EISA_PRODUCT_ASU5201	"Asus P/E-P55T2P4D Main Board (CFG File V1.2)"
 
/* ATI products */
#define	EISA_PRODUCT_ATI1500	"ATI AT-1500 Ethernet"
#define	EISA_PRODUCT_ATI1700	"ATI AT-1700 Ethernet"
#define	EISA_PRODUCT_ATI4400	"ATI mach32 Graphics"
#define	EISA_PRODUCT_ATI4402	"ATI mach32 Graphics"
#define	EISA_PRODUCT_ATI4410	"ATI mach32 Graphics"
#define	EISA_PRODUCT_ATI4420	"ATI mach32 Graphics"
#define	EISA_PRODUCT_ATI4430	"ATI mach32 VLB Graphics"

#define	EISA_PRODUCT_ATT2402	"AT&T SCSI (StarServer E)"
#define	EISA_PRODUCT_ATT2404	"DPT SCSI (PM2012B/9X)"

#define	EISA_PRODUCT_AVI2E01	"AVIEW2E SVGA"

#define	EISA_PRODUCT_AVM0001	"ISDN-Controller A1"

/* Banyan Vines products */
#define	EISA_PRODUCT_BAN0440	"Banyan Vines ICA"
#define	EISA_PRODUCT_BAN0670	"Banyan Vines ICAplus"
#define	EISA_PRODUCT_BAN0680	"Banyan Vines ICA/RM"

/* BusLogic products */
#define	EISA_PRODUCT_BUS4201	"BusLogic Bt74xB SCSI"
#define	EISA_PRODUCT_BUS4202	"BusLogic Bt74xC SCSI"
#define	EISA_PRODUCT_BUS6001	"BusLogic Bt760 Ethernet"
#define	EISA_PRODUCT_BUS6301	"BusLogic Bt763E 82596-based Ethernet"

/* Cache Computers products */
#define	EISA_PRODUCT_CCI0000	"Cache Computers, Inc. Memory Refresh Controller"
#define	EISA_PRODUCT_CCI1001	"Cache Computers, Inc. 486/25 System Board"
#define	EISA_PRODUCT_CCI2001	"Cache Computers, Inc. 486/33 System Board"
#define	EISA_PRODUCT_CCI3001	"Cache Computers, Inc. Modular System Board"
#define	EISA_PRODUCT_CCI3009	"Cache Computers, Inc. Modular System Board"
#define	EISA_PRODUCT_CCI4000	"Cache Computers, Inc. 486/33 CPU Board"
#define	EISA_PRODUCT_CCI4001	"Cache Computers, Inc. 486/33 CPU Board"
#define	EISA_PRODUCT_CCI5001	"Cache Computers, Inc. 486/50 CPU Board"
#define	EISA_PRODUCT_CCI6001	"Cache Computers, Inc. LAN Adapter"
#define	EISA_PRODUCT_CCI7001	"Cache Computers, Inc. System Board"
#define	EISA_PRODUCT_CCI8001	"Cache Computers, Inc. System Board"

/* Chase products */
#define	EISA_PRODUCT_CHAA041	"Chase AT4 Intelligent Serial Controller"
#define	EISA_PRODUCT_CHAA081	"Chase AT8 Intelligent Serial Controller"
#define	EISA_PRODUCT_CHAA091	"Chase AT8+ Intelligent Serial Controller"
#define	EISA_PRODUCT_CHAA161	"Chase AT16 Intelligent Serial Controller"
#define	EISA_PRODUCT_CHAA171	"Chase AT16+ Intelligent Serial Controller"

/* CMD Technology products */
#define	EISA_PRODUCT_CMD0003	"CMD Technology, Inc. SCSI Host Adapter"

#define	EISA_PRODUCT_CNT2000	"900E/950E 32-bit Ethernet"

/* Cogent products */
#define	EISA_PRODUCT_COG5000	"Cogent eMASTER+ AT Combo 16-Bit Workstation Ethernet"
#define	EISA_PRODUCT_COG7002	"Cogent eMASTER+ ATS Combo Bus-Mastering Ethernet"
#define	EISA_PRODUCT_COG9002	"Cogent eMASTER+ XL 32-Bit Burst-mode Ethernet"

/* Compaq products */
#define	EISA_PRODUCT_CPQ0101	"Compaq SYSTEMPRO System Board"
#define	EISA_PRODUCT_CPQ0109	"Compaq SYSTEMPRO System Board (ASSY (001981))"
#define	EISA_PRODUCT_CPQ0401	"Compaq DESKPRO 486/33L or 386/33L System Board"
#define	EISA_PRODUCT_CPQ0501	"Compaq DESKPRO/M System Board"
#define	EISA_PRODUCT_CPQ0509	"Compaq DESKPRO/M System Board with Audio"
#define	EISA_PRODUCT_CPQ0511	"Compaq SYSTEMPRO/LT System Board"
#define	EISA_PRODUCT_CPQ0601	"Compaq ProSignia Server"
#define	EISA_PRODUCT_CPQ0609	"Compaq ProSignia Server"
#define	EISA_PRODUCT_CPQ0611	"Compaq ProSignia Server"
#define	EISA_PRODUCT_CPQ0621	"Compaq ProSignia Server (ASSY (3154))"
#define	EISA_PRODUCT_CPQ0629	"Compaq ProSignia Server (ASSY (3154))"
#define	EISA_PRODUCT_CPQ0631	"Compaq ProLiant 1000 Server"
#define	EISA_PRODUCT_CPQ0639	"Compaq ProLiant 1000 Server"
#define	EISA_PRODUCT_CPQ1001	"Compaq Portable 486"
#define	EISA_PRODUCT_CPQ1009	"Compaq Portable 486/66"
#define	EISA_PRODUCT_CPQ1201	"Compaq DESKPRO 486/25"
#define	EISA_PRODUCT_CPQ1301	"Compaq DESKPRO 486/50L"
#define	EISA_PRODUCT_CPQ1401	"Compaq Portable 486c"
#define	EISA_PRODUCT_CPQ1409	"Compaq Portable 486c/66"
#define	EISA_PRODUCT_CPQ1501	"Compaq SYSTEMPRO/XL Server"
#define	EISA_PRODUCT_CPQ1509	"Compaq ProLiant 4000 Server"
#define	EISA_PRODUCT_CPQ1519	"Compaq ProLiant 2000 Server"
#define	EISA_PRODUCT_CPQ3001	"Compaq Advanced VGA"
#define	EISA_PRODUCT_CPQ3011	"Compaq QVision 1024/E Video Controller"
#define	EISA_PRODUCT_CPQ3021	"Compaq QVision 1024/I Video Controller"
#define	EISA_PRODUCT_CPQ3111	"Compaq QVision 1024/E Graphics Controller for Alpha AXP"
#define	EISA_PRODUCT_CPQ3112	"Compaq QVision 1280/E Graphics Controller for Alpha AXP"
#define	EISA_PRODUCT_CPQ3121	"Compaq QVision 1024/I Graphics Controller"
#define	EISA_PRODUCT_CPQ3122	"Compaq QVision 1280/I Graphics Controller"
#define	EISA_PRODUCT_CPQ4001	"Compaq 32-Bit Intelligent Drive Array Controller"
#define	EISA_PRODUCT_CPQ4002	"Compaq Intelligent Drive Array Controller-2"
#define	EISA_PRODUCT_CPQ4010	"Compaq 32-Bit Intelligent Drive Array Expansion Controller"
#define	EISA_PRODUCT_CPQ4020	"Compaq SMART Array Controller"
#define	EISA_PRODUCT_CPQ4300	"Compaq Advanced ESDI Fixed Disk Controller"
#define	EISA_PRODUCT_CPQ4401	"Compaq Integrated SCSI-2 Options Port"
#define	EISA_PRODUCT_CPQ4410	"Compaq Integrated 32-Bit Fast-SCSI-2 Controller"
#define	EISA_PRODUCT_CPQ4411	"Compaq 32-Bit Fast-SCSI-2 Controller"
#define	EISA_PRODUCT_CPQ4420	"Compaq 6260 SCSI-2 Controller"
#define	EISA_PRODUCT_CPQ5000	"Compaq 386/33 System Processor Board used as Secondary"
#define	EISA_PRODUCT_CPQ5281	"Compaq 486/50 System Processor Board used as Secondary"
#define	EISA_PRODUCT_CPQ5282	"Compaq 486/50 System Processor Board used as Secondary"
#define	EISA_PRODUCT_CPQ5287	"Compaq 5/66 System Processor Board used as Secondary"
#define	EISA_PRODUCT_CPQ528F	"Compaq 486DX2/66 System Processor Board used as Secondary"
#define	EISA_PRODUCT_CPQ5900	"Compaq 486/33 System Processor Board used as Secondary"
#define	EISA_PRODUCT_CPQ5A00	"Compaq 486/33 System Processor Board (ASSY (002013) used as Secondary)"
#define	EISA_PRODUCT_CPQ5B00	"Compaq 486DX2/66 System Processor Board used as Secondary"
#define	EISA_PRODUCT_CPQ5C00	"Compaq 486/33 System Processor Board used as Secondary"
#define	EISA_PRODUCT_CPQ6000	"Compaq 32-Bit DualSpeed Token Ring Controller"
#define	EISA_PRODUCT_CPQ6001	"Compaq 32-Bit DualSpeed Token Ring Controller"
#define	EISA_PRODUCT_CPQ6100	"Compaq 32-Bit NetFlex Controller"
#define	EISA_PRODUCT_CPQ6101	"Compaq NetFlex-2 ENET-TR"
#define	EISA_PRODUCT_CPQ6200	"Compaq DualPort Ethernet Controller"
#define	EISA_PRODUCT_CPQ7000	"Compaq 32-Bit Server Manager/R Board"
#define	EISA_PRODUCT_CPQ7001	"Compaq 32-Bit Server Manager/R Board"
#define	EISA_PRODUCT_CPQ9004	"Compaq 386/33 Processor Board"
#define	EISA_PRODUCT_CPQ9005	"Compaq 386/25 Processor Board"
#define	EISA_PRODUCT_CPQ9013	"Compaq 486DX2/66 System Processor Board used as Primary"
#define	EISA_PRODUCT_CPQ9014	"Compaq 486/33 System Processor Board used as Primary"
#define	EISA_PRODUCT_CPQ9015	"Compaq 486/33 Processor Board"
#define	EISA_PRODUCT_CPQ9016	"Compaq 486DX2/66 Processor Board"
#define	EISA_PRODUCT_CPQ9017	"Compaq 486DX2/50 Processor Board"
#define	EISA_PRODUCT_CPQ9018	"Compaq 486/33 Processor Board (8 MB)"
#define	EISA_PRODUCT_CPQ9034	"Compaq 486SX/25 Processor Board"
#define	EISA_PRODUCT_CPQ9035	"Compaq 486SX/16 Processor Board"
#define	EISA_PRODUCT_CPQ9036	"Compaq 486SX/25 Processor Board (8 MB)"
#define	EISA_PRODUCT_CPQ9037	"Compaq 486SX/16 Processor Board (8 MB)"
#define	EISA_PRODUCT_CPQ9038	"Compaq 486SX/33 Processor Board (8 MB)"
#define	EISA_PRODUCT_CPQ903C	"Compaq 486SX/33 Processor Board (4 MB)"
#define	EISA_PRODUCT_CPQ9040	"Compaq 5/66 Processor Board"
#define	EISA_PRODUCT_CPQ9041	"Compaq 5/66 Processor Board"
#define	EISA_PRODUCT_CPQ9042	"Compaq 5/66 Processor Board"
#define	EISA_PRODUCT_CPQ9043	"Compaq 5/66 Processor Board"
#define	EISA_PRODUCT_CPQ9044	"Compaq 5/60 Processor Board"
#define	EISA_PRODUCT_CPQ9045	"Compaq 5/60 Processor Board"
#define	EISA_PRODUCT_CPQ9046	"Compaq 5/60 Processor Board"
#define	EISA_PRODUCT_CPQ9047	"Compaq 5/60 Processor Board"
#define	EISA_PRODUCT_CPQ9281	"Compaq 486/50 System Processor Board used as Primary"
#define	EISA_PRODUCT_CPQ9282	"Compaq 486/50 System Processor Board used as Primary"
#define	EISA_PRODUCT_CPQ9287	"Compaq 5/66 System Processor Board used as Primary"
#define	EISA_PRODUCT_CPQ928F	"Compaq 486DX2/66 System Processor Board used as Primary"
#define	EISA_PRODUCT_CPQ9381	"Compaq 486/50 System Processor Board"
#define	EISA_PRODUCT_CPQ9382	"Compaq 486/50 System Processor Board"
#define	EISA_PRODUCT_CPQ9387	"Compaq 5/66 System Processor Board"
#define	EISA_PRODUCT_CPQ9481	"Compaq 486/50 System Processor Board"
#define	EISA_PRODUCT_CPQ9482	"Compaq 486/50 System Processor Board"
#define	EISA_PRODUCT_CPQ9487	"Compaq 5/66 System Processor Board"
#define	EISA_PRODUCT_CPQ9901	"Compaq 486SX/16 Processor Board"
#define	EISA_PRODUCT_CPQ9902	"Compaq 486SX/16 Processor Board"
#define	EISA_PRODUCT_CPQ9903	"Compaq 486SX/25 Processor Board"
#define	EISA_PRODUCT_CPQ9904	"Compaq 486SX/25 Processor Board"
#define	EISA_PRODUCT_CPQ9905	"Compaq 486SX/25 Processor Board"
#define	EISA_PRODUCT_CPQ9906	"Compaq 486/33 Processor Board"
#define	EISA_PRODUCT_CPQ9907	"Compaq 486DX2/66 Processor Board"
#define	EISA_PRODUCT_CPQ9908	"Compaq 486SX/16 Processor Board"
#define	EISA_PRODUCT_CPQ9909	"Compaq 486SX/16 Processor Board"
#define	EISA_PRODUCT_CPQ990A	"Compaq 486SX/25 Processor Board"
#define	EISA_PRODUCT_CPQ990B	"Compaq 486SX/25 Processor Board"
#define	EISA_PRODUCT_CPQ990C	"Compaq 486SX/25 Processor Board"
#define	EISA_PRODUCT_CPQ990D	"Compaq 486/33 Processor Board"
#define	EISA_PRODUCT_CPQ990E	"Compaq 486SX/33 Processor Board (8 MB)"
#define	EISA_PRODUCT_CPQ990F	"Compaq 486SX/33 Processor Board (8 MB)"
#define	EISA_PRODUCT_CPQ9990	"Compaq 386/33 System Processor Board used as Primary"
#define	EISA_PRODUCT_CPQ9991	"Compaq 386/33 Desktop Processor Board"
#define	EISA_PRODUCT_CPQ9999	"Compaq 486/33 System Processor Board used as Primary"
#define	EISA_PRODUCT_CPQ999A	"Compaq 486/33 Desktop Processor Board"
#define	EISA_PRODUCT_CPQA000	"Compaq Enhanced Option Slot Serial Board"
#define	EISA_PRODUCT_CPQA010	"Compaq Enhanced Option Slot Modem Board"
#define	EISA_PRODUCT_CPQA020	"Compaq Integrated CD Rom Adapter"
#define	EISA_PRODUCT_CPQA030	"Compaq Integrated CD Rom Adapter"
#define	EISA_PRODUCT_CPQF000	"Compaq Fixed Disk Drive Feature"
#define	EISA_PRODUCT_CPQF100	"Compaq Ethernet 16TP Controller"
#define	EISA_PRODUCT_CPQFA0D	"Compaq SYSTEMPRO 4-Socket System Memory Board"
#define	EISA_PRODUCT_CPQFA0E	"Compaq SYSTEMPRO 6-Socket System Memory Board"
#define	EISA_PRODUCT_CPQFA0F	"Compaq DESKPRO 486/25 System Memory Board"
#define	EISA_PRODUCT_CPQFA1A	"Compaq DESKPRO 3-Socket System Memory Board"
#define	EISA_PRODUCT_CPQFA1B	"Compaq DESKPRO 486/50 System Memory Board"
#define	EISA_PRODUCT_CPQFA1C	"Compaq System Memory Expansion Board"
#define	EISA_PRODUCT_CPQFA1D	"Compaq SYSTEMPRO/XL Memory Expansion Board"
#define	EISA_PRODUCT_CPQFB03	"Compaq Async/Parallel Printer Intf Assy 000990"
#define	EISA_PRODUCT_CPQFB07	"Compaq DESKPRO 2400 Baud Modem"
#define	EISA_PRODUCT_CPQFC0B	"Compaq Advanced Graphics 1024 Board"
#define	EISA_PRODUCT_CPQFD08	"Compaq 135Mb, 150/250Mb Tape Adapter"
#define	EISA_PRODUCT_CPQFD13	"Compaq 15MHz ESDI Fixed Disk Controller 001283"
#define	EISA_PRODUCT_CPQFD17	"Compaq SCSI Tape Adapter"

#define	EISA_PRODUCT_CPX0301	"Universal 10/100VG Ethernet"

#define	EISA_PRODUCT_CRS3203	"Crescendo 320 FDDI/CDDI"
#define	EISA_PRODUCT_CRS3204	"Cisco CDDI/FDDI"

#define	EISA_PRODUCT_CSI0690	"F70X9 FDDI DNI"

#define	EISA_PRODUCT_CUI0000	"Examples -- Virtual Board"

/* Digi products */
#define	EISA_PRODUCT_DBI0101	"Digi C/X Host Adapter"
#define	EISA_PRODUCT_DBI0102	"Digi C/X Host Adapter"
#define	EISA_PRODUCT_DBI0201	"Digi EISA/Xem Host Adapter"
#define	EISA_PRODUCT_DBI0301	"Digi EPC/X Host Adapter"
#define	EISA_PRODUCT_DBI0501	"Digi Ports/Xem Host Adapter"
#define	EISA_PRODUCT_DBI0601	"Digi EPC/X Host Adapter"
#define	EISA_PRODUCT_DBI0701	"Digi C/X Host Adapter"
#define	EISA_PRODUCT_DBI0801	"Digi PC/Xr"
#define	EISA_PRODUCT_DBI0901	"Digi PC/Xt"
#define	EISA_PRODUCT_DBI0C01	"Digi DataFire"
#define	EISA_PRODUCT_DBI0D01	"Digi DataFire/4"
#define	EISA_PRODUCT_DBI0E01	"Digi PC IMAC"
#define	EISA_PRODUCT_DBI0F01	"Digi PC IMAC/4"
#define	EISA_PRODUCT_DBI1001	"Digi PC/Xe"
#define	EISA_PRODUCT_DBI1101	"Digi ES/4 Host Adapter"
#define	EISA_PRODUCT_DBI1201	"Digi Acceleport Xr 920"

/* Digital Equipment products */
#define	EISA_PRODUCT_DEC1011	"Digital Equipment EVC-1 Video"
#define	EISA_PRODUCT_DEC1021	"Digital Equipment ESC-1 SCSI"
#define	EISA_PRODUCT_DEC1031	"Digital Equipment DECpc MTE Series System Board"
#define	EISA_PRODUCT_DEC2030	"Digital Equipment DE203 Ethernet"
#define	EISA_PRODUCT_DEC2040	"Digital Equipment DE204 Ethernet"
#define	EISA_PRODUCT_DEC2050	"Digital Equipment DE205 Ethernet"
#define	EISA_PRODUCT_DEC2400	"Digital Equipment DECpc AXP/150 System Board"
#define	EISA_PRODUCT_DEC2500	"Digital Equipment DEC EISA NVRAM for Alpha AXP"
#define	EISA_PRODUCT_DEC2A01	"Digital Equipment AlphaServer 2100 Family System Board"
#define	EISA_PRODUCT_DEC2E00	"Digital Equipment KFESA DSSI Host Adapter"
#define	EISA_PRODUCT_DEC2F00	"Digital Equipment DNSES WANcontroller"
#define	EISA_PRODUCT_DEC3001	"Digital Equipment DEFEA FDDI Controller"
#define	EISA_PRODUCT_DEC3002	"Digital Equipment DEFEA FDDI Controller"
#define	EISA_PRODUCT_DEC3003	"Digital Equipment DEFEA FDDI Controller"
#define	EISA_PRODUCT_DEC3004	"Digital Equipment DEFEA FDDI Controller"
#define	EISA_PRODUCT_DEC4220	"Digital Equipment DE422-SA Ethernet"
#define	EISA_PRODUCT_DEC4250	"Digital Equipment DE425-AA Ethernet"
#define	EISA_PRODUCT_DEC5000	"Digital Equipment AlphaServer 1000 System Board"
#define	EISA_PRODUCT_DEC5100	"Digital Equipment AlphaStation 600 System Board"
#define	EISA_PRODUCT_DEC5301	"Digital Equipment 800 AlphaServer System Board"
#define	EISA_PRODUCT_DEC6000	"Digital Equipment AlphaServer 8200 System Board"
#define	EISA_PRODUCT_DEC6400	"Digital Equipment AlphaServer 4100 System Board"
#define	EISA_PRODUCT_DEC8101	"Digital Equipment VGA 1024 Graphics"
#define	EISA_PRODUCT_DEC8102	"Digital Equipment 8514/A-Compatible Graphics"
#define	EISA_PRODUCT_DEC8103	"Digital Equipment VGA 1024 NI Graphics"
#define	EISA_PRODUCT_DEC8300	"Digital Equipment DEPCA LC Ethernet"
#define	EISA_PRODUCT_DEC8301	"Digital Equipment DEPCA TURBO Ethernet"

/* Dell products */
#define	EISA_PRODUCT_DEL0000	"Dell Generic ISA Board"
#define	EISA_PRODUCT_DEL0001	"Dell System(R) 425E(TM) System Board"
#define	EISA_PRODUCT_DEL0002	"Dell System(R) 433E(TM) System Board"
#define	EISA_PRODUCT_DEL0003	"Dell System(R) 425TE(TM) System Board"
#define	EISA_PRODUCT_DEL0004	"Dell System(R) 433TE(TM) System Board"
#define	EISA_PRODUCT_DEL0005	"Dell Powerline(TM) Workstation 433DE(TM) System Board"
#define	EISA_PRODUCT_DEL0006	"Dell Powerline(TM) Workstation 420DE(TM) System Board"
#define	EISA_PRODUCT_DEL0007	"Dell Powerline(TM) Workstation 450DE(TM) System Board"
#define	EISA_PRODUCT_DEL0008	"Dell Powerline(TM) Server 433SE(TM) System Board"
#define	EISA_PRODUCT_DEL0009	"Dell Powerline(TM) Server 420SE(TM) System Board"
#define	EISA_PRODUCT_DEL000A	"Dell Powerline(TM) Server 450SE(TM) System Board"
#define	EISA_PRODUCT_DEL000B	"Dell PowerLine(TM) Workstation 425DE(TM) System Board"
#define	EISA_PRODUCT_DEL000C	"Dell PowerLine(TM) Server 425SE(TM) System Board"
#define	EISA_PRODUCT_DEL0011	"Dell Powerline(TM) Server 450SE/2(TM) System Board"
#define	EISA_PRODUCT_DEL0019	"Dell Powerline(TM) Server 466SE(TM) System Board"
#define	EISA_PRODUCT_DEL0021	"Dell Powerline(TM) Workstation 450DE/2(TM) System Board"
#define	EISA_PRODUCT_DEL0029	"Dell Powerline(TM) Workstation 466DE(TM) System Board"
#define	EISA_PRODUCT_DEL002A	"Dell Powerline(TM) Workstation P60/DE(TM) System Board"
#define	EISA_PRODUCT_DEL002B	"Dell Powerline(TM) Workstation P66/DE(TM) System Board"
#define	EISA_PRODUCT_DEL002C	"Dell Powerline(TM) Workstation P60/SE(TM) System Board"
#define	EISA_PRODUCT_DEL002D	"Dell Powerline(TM) Workstation P66/SE(TM) System Board"
#define	EISA_PRODUCT_DEL0031	"Dell 486/ME System Board"
#define	EISA_PRODUCT_DEL0036	"Dell 406x/XE System Board"
#define	EISA_PRODUCT_DEL0038	"Dell 456x/XE System Board"
#define	EISA_PRODUCT_DEL0054	"Dell System PowerEdge 2100"
#define	EISA_PRODUCT_DEL0058	"Dell System PowerEdge 4100"
#define	EISA_PRODUCT_DEL005A	"Dell System PowerEdge 2200"
#define	EISA_PRODUCT_DEL005C	"Dell System PowerEdge 4200"
#define	EISA_PRODUCT_DEL2100	"Dell Remote Server Assistant Card"
#define	EISA_PRODUCT_DEL4001	"Dell Drive Array"
#define	EISA_PRODUCT_DEL4002	"Dell SCSI Array Controller"
#define	EISA_PRODUCT_DEL6001	"Dell DGX Video Subsystem"
#define	EISA_PRODUCT_DELFC00	"Dell GPX-1024 Graphics Performance Accelerator"
#define	EISA_PRODUCT_DELFC01	"Dell VGA Professional 16-bit"
#define	EISA_PRODUCT_DELFC02	"Dell Paradise Hi-Res Graphics Adapter"
#define	EISA_PRODUCT_DELFC03	"Dell Paradise Hi-Res Graphics Card"
#define	EISA_PRODUCT_DELFD00	"Dell UltraStor 12F/12F-24 ESDI/Diskette Cntrl"
#define	EISA_PRODUCT_DELFD02	"Dell Archive XL Tape Host Adapter"
#define	EISA_PRODUCT_DELFD03	"Dell Wangtek Tape Host Adapter"
#define	EISA_PRODUCT_DELFD05	"Dell Adaptec AHA-1510 ISA SCSI Host Adapter"

#define	EISA_PRODUCT_DIS0000	"NETSERVER LH PRO"

/* DPT products */
#define	EISA_PRODUCT_DPT2402	"DPT PM2012A/9X SCSI"
#define	EISA_PRODUCT_DPT2403	"DPT PM2012A/90 SCSI"
#define	EISA_PRODUCT_DPTA401	"DPT PM2012B/9X SCSI"
#define	EISA_PRODUCT_DPTA402	"DPT PM2012B2/9X SCSI (Banyan Vines)"
#define	EISA_PRODUCT_DPTA410	"DPT PM2X22A/9X SCSI"
#define	EISA_PRODUCT_DPTA501	"DPT PM2012B1/9X SCSI"
#define	EISA_PRODUCT_DPTA502	"DPT PM2012B2/9X SCSI"
#define	EISA_PRODUCT_DPTA701	"DPT PM2011B1/9X SCSI"
#define	EISA_PRODUCT_DPTBC01	"DPT PM3011/7X ESDI"

#define	EISA_PRODUCT_DTC1101	"DTC2290 IDE"
#define	EISA_PRODUCT_DTC3101	"DTC3290 SCSI"

/* DeskStation Technology, Inc. products */
#define	EISA_PRODUCT_DTI0000	"DeskStation Technology, Inc. Evolution RISC PC"
#define	EISA_PRODUCT_DTI2000	"DeskStation Technology, Inc. ESP2000A/ESP2000 System Processor Board"
#define	EISA_PRODUCT_DTI2002	"DeskStation Technology, Inc. ESP2002 Integrated System Processor"

/* DTK products */
#define	EISA_PRODUCT_DTK0001	"DTK PLM-3300I 80486 Board"
#define	EISA_PRODUCT_DTK0003	"DTK PLM-3331P EISACACHE486 33/25/50 MHZ"

#define	EISA_PRODUCT_ECS0580	"DI-580A SCSI"
#define	EISA_PRODUCT_ECS0590	"DI-590 SCSI"

/* ELSA products */
#define	EISA_PRODUCT_ELS8041	"ELSA WINNER 1000 Enhanced VGA"

/* ETI products */
#define	EISA_PRODUCT_ETI1001	"ETI NE3300 Ethernet Rev. C & D"

#define	EISA_PRODUCT_EVX0002	"PN-3000 System Board"

#define	EISA_PRODUCT_FCT0001	"SYSTEM BOARD"
#define	EISA_PRODUCT_FCT0002	"386 SYSTEM BOARD"
#define	EISA_PRODUCT_FCT0003	"486 SYSTEM BOARD"

#define	EISA_PRODUCT_FIC0000	"LEO 486VE Main Board"

#define	EISA_PRODUCT_FIX1516	"15-16MB Memory Hole Patch - Netserver LF/LC 5/66"

/* FORE Systems products */
#define	EISA_PRODUCT_FSI2001	"FORE Systems ESA-200 ATM"
#define	EISA_PRODUCT_FSI2002	"FORE Systems ESA-200A ATM"
#define	EISA_PRODUCT_FSI2003	"FORE Systems ESA-200E ATM"

/* GDT products */
#define	EISA_PRODUCT_GDT2001	"GDT GDT2000/GDT2020 Fast-SCSI Cache Controller - Rev. 1.0"
#define	EISA_PRODUCT_GDT3001	"GDT GDT3000/GDT3020 Dual Channel SCSI Controller - Rev. 1.0"
#define	EISA_PRODUCT_GDT3002	"GDT GDT30x0A Cache Controller"
#define	EISA_PRODUCT_GDT3003	"GDT GDT3000B/GDT3010A SCSI Cache Controller - Rev. 1.0"

#define	EISA_PRODUCT_GIT0000	"G486PEL EISA & LOCAL Bus Mother Board."
#define	EISA_PRODUCT_GIT0001	"G486HVL EISA & VESA LOCAL Bus Mother Board."

#define	EISA_PRODUCT_HCL0801	"HP Limited PANTHER System Board"

#define	EISA_PRODUCT_HIT0001	"MCC Mini-EISA486 Board"

#define	EISA_PRODUCT_HKG0011	"Distributed Signal Conditioning Front-End"

#define	EISA_PRODUCT_HMS0000	"HMSI ESIC EVALUATION BOARD"

/* HP products */
#define	EISA_PRODUCT_HWP0000	"Hewlett-Packard Monochrome Plus Video Board (35732A)"
#define	EISA_PRODUCT_HWP0010	"Hewlett-Packard Multimode Video Adapter (45981A)"
#define	EISA_PRODUCT_HWP0020	"Hewlett-Packard Multimode Color Adapter Board (45984A)"
#define	EISA_PRODUCT_HWP0030	"Hewlett-Packard Enhanced Graphics Adapter Board (45983A)"
#define	EISA_PRODUCT_HWP0070	"Hewlett-Packard Intelligent Graphics Controller 20 (A1083A)"
#define	EISA_PRODUCT_HWP0C70	"Hewlett-Packard HP-IB Host Adapter"
#define	EISA_PRODUCT_HWP0C80	"Hewlett-Packard SCSI Host Adapter (Cirrus-II) -- 25525A"
#define	EISA_PRODUCT_HWP1400	"Hewlett-Packard Dual Serial Interface Board (24541B)"
#define	EISA_PRODUCT_HWP1410	"Hewlett-Packard Internal 2400 Baud Modem (24551A)"
#define	EISA_PRODUCT_HWP1420	"Hewlett-Packard Internal 1200 Baud Modem (24550A)"
#define	EISA_PRODUCT_HWP1440	"Hewlett-Packard Terminal Multiplexor Board (D2040A)"
#define	EISA_PRODUCT_HWP1450	"Hewlett-Packard HP-IB Interface board (82335A)"
#define	EISA_PRODUCT_HWP1460	"Hewlett-Packard ScanJet Plus Interface (88290A)"
#define	EISA_PRODUCT_HWP1461	"Hewlett-Packard ScanJet Plus Interface (88290A)"
#define	EISA_PRODUCT_HWP1510	"Hewlett-Packard SS8-E MUX"
#define	EISA_PRODUCT_HWP1801	"Hewlett-Packard StarLAN 10 PC Link II (27240A)"
#define	EISA_PRODUCT_HWP1810	"Hewlett-Packard ThinLAN Interface Board (27210B)"
#define	EISA_PRODUCT_HWP1811	"Hewlett-Packard ThinLAN PC Adapter Card (27250A)"
#define	EISA_PRODUCT_HWP1820	"Hewlett-Packard EtherTwist Adapter Card/8 (27245-60002)"
#define	EISA_PRODUCT_HWP1832	"Hewlett-Packard EtherTwist PC LAN Adapter/16 TP Plus (27247B)"
#define	EISA_PRODUCT_HWP1840	"Hewlett-Packard EtherTwist LAN Adapter/32"
#define	EISA_PRODUCT_HWP1850	"Hewlett-Packard LAN AdapterCard -- 25567A"
#define	EISA_PRODUCT_HWP1870	"Hewlett-Packard PSI"
#define	EISA_PRODUCT_HWP18A0	"Hewlett-Packard EtherTwist PC LAN Adapter/16 TL Plus (27252A)"
#define	EISA_PRODUCT_HWP18C0	"Hewlett-Packard EtherTwist PC LAN Adapter NC/16 TP (J2405A)"
#define	EISA_PRODUCT_HWP18E0	"Hewlett-Packard 100Mbps ATM Card"
#define	EISA_PRODUCT_HWP1940	"Hewlett-Packard 10/100VG Selectable LAN Adapter (J2577A)"
#define	EISA_PRODUCT_HWP1980	"Hewlett-Packard ATM Adapter -- J2802A"
#define	EISA_PRODUCT_HWP1990	"Hewlett-Packard 100VG AnyLAN adapter card"
#define	EISA_PRODUCT_HWP19A0	"Hewlett-Packard Low-cost PSI"
#define	EISA_PRODUCT_HWP1C00	"Hewlett-Packard Serial/Parallel Interface Board (24540B)"
#define	EISA_PRODUCT_HWP2002	"Hewlett-Packard ScanJet II Interface (C1752A)"
#define	EISA_PRODUCT_HWP2051	"Hewlett-Packard Test Adapter Card"
#define	EISA_PRODUCT_HWP2080	"Hewlett-Packard ScanJet II Interface (C2502A)"
#define	EISA_PRODUCT_HWPC000	"Hewlett-Packard Series 700 System Board"
#define	EISA_PRODUCT_HWPC010	"Hewlett-Packard Series 700 System Board"
#define	EISA_PRODUCT_HWPC051	"Hewlett-Packard Series 700 System Board"
#define	EISA_PRODUCT_HWPC091	"Hewlett-Packard System Board"
#define	EISA_PRODUCT_HWPC0D1	"Hewlett-Packard System Board"
#define	EISA_PRODUCT_HWPC0E1	"Hewlett-Packard System Board"

/* IBM products */
#define	EISA_PRODUCT_IBM0001	"Auto 16/4 Token Ring"
#define	EISA_PRODUCT_IBM1000	"16/4 Busmaster TR"
#define	EISA_PRODUCT_IBM1060	"100/10 ISA Ethernet"
#define	EISA_PRODUCT_IBM1061	"100/10 ISA Ethernet"

#define	EISA_PRODUCT_ICLA080	"EtherTeam 32 Ethernet"

#define	EISA_PRODUCT_IDS0100	"EISC960 Caching SCSI"

#define	EISA_PRODUCT_IIN0B01	"Intel TokenExpress(tm) ISA/8"

/* IKON products */
#define	EISA_PRODUCT_IKN1110	"IKON hardcopy boards: 10092, 10097, or 10111"

/* Integrated Micro Solution Inc. products */
#define	EISA_PRODUCT_IMS1001	"Integrated Micro Solution Inc. 486 System Board"

#define	EISA_PRODUCT_ING2040	"HCL-Hewlett Packard Limited PANTHER System Board"

/* Interphase products */
#define	EISA_PRODUCT_INP0010	"Interphase Barracuda E/4810"
#define	EISA_PRODUCT_INP25D0	"Interphase Seahawk 4811 FDDI Controller"
#define	EISA_PRODUCT_INP5000	"Interphase 4800 EISA-PCI Bridge"

/* Intel products */
#define	EISA_PRODUCT_INT0000	"Intel Mercury/Neptune PCI-EISA Main Board"
#define	EISA_PRODUCT_INT0081	"Intel HP Limited PANTHER System Board"
#define	EISA_PRODUCT_INT0701	"Intel TokenExpress(tm) ISA 16/4"
#define	EISA_PRODUCT_INT0703	"Intel TokenExpress(tm) ISA/16s"
#define	EISA_PRODUCT_INT0902	"Intel TokenExpress(tm) 16/4"
#define	EISA_PRODUCT_INT0B01	"Intel TokenExpress(tm) ISA/8"
#define	EISA_PRODUCT_INT1000	"Intel EtherExpress 16 Family Ethernet"
#define	EISA_PRODUCT_INT1010	"Intel EtherExpress 32 Flash Ethernet"
#define	EISA_PRODUCT_INT1030	"Intel EtherExpress(TM) PRO/10 Ethernet"
#define	EISA_PRODUCT_INT1031	"Intel EtherExpress(TM) PRO/10+ Ethernet"
#define	EISA_PRODUCT_INT1060	"Intel EtherExpress(tm) PRO/100 Ethernet"
#define	EISA_PRODUCT_INT1201	"Intel TokenExpress(tm) EISA/32"
#define	EISA_PRODUCT_INT3061	"Intel LP486E System Board"
#define	EISA_PRODUCT_INT3079	"Intel X-Series Desktop System Board"
#define	EISA_PRODUCT_INT3089	"Intel X-Series Deskside System Board"
#define	EISA_PRODUCT_INT30A1	"Intel L486 Series System Board"
#define	EISA_PRODUCT_INT30A9	"Intel L486 Series System Board"
#define	EISA_PRODUCT_INT30D1	"Intel X-Series System Board"
#define	EISA_PRODUCT_INT30F1	"Intel X-Series Premium System Board"
#define	EISA_PRODUCT_INT3132	"Intel ECC Memory Module (BXECCMEM0)"
#define	EISA_PRODUCT_INT31A0	"Intel System Board"

#define	EISA_PRODUCT_ISY0010	"(SYSTEM) VGA Board"
#define	EISA_PRODUCT_ISY0020	"(SYSTEM) COM Ports"
#define	EISA_PRODUCT_ISY0030	"(SYSTEM) Mother Board PS-2 style Mouse"
#define	EISA_PRODUCT_ISY0040	"(SYSTEM) Hard Disk Controller"
#define	EISA_PRODUCT_ISY0050	"(SYSTEM) Floppy Drive Controller"
#define	EISA_PRODUCT_ISY0060	"(SYSTEM) LPT Ports"
#define	EISA_PRODUCT_ISY0070	"(SYSTEM) IRQ9 Cascaded Interrupt"

#define	EISA_PRODUCT_ITC0001	"EISA-486C Main Board"

#define	EISA_PRODUCT_ITK0011	"ixEins Basic S0/Up0 ISDN Version 1.1"
#define	EISA_PRODUCT_ITK0012	"ixEins Basic S0/Up0 ISDN Version 1.2"

#define	EISA_PRODUCT_KCI3201	"ET32EM 32-bit BUS-MASTER Ethernet"
#define	EISA_PRODUCT_KCI3202	"ET-32EM 32-bit Bus Master Ethernet"

/* LeafNET products */
#define	EISA_PRODUCT_LEF8000	"LeafNET Ethernet"

/* Madge products */
#define	EISA_PRODUCT_MDG0002	"Madge Networks Smart 16/4 Ringnode"
#define	EISA_PRODUCT_MDG0010	"Madge Networks Smart 16/4 AT Ringnode"
#define	EISA_PRODUCT_MDG0020	"Madge Networks Smart 16/4 ISA Client Ringnode"
#define	EISA_PRODUCT_MDG2000	"Madge Networks Blue+ 16/4 ISA Token Ring"
#define	EISA_PRODUCT_MDG2010	"Madge Networks Blue+ 16/4 ISA PnP Token Ring"

#define	EISA_PRODUCT_MCC0000	"EISA-486 System Board"
#define	EISA_PRODUCT_MCC0001	"IISA-486 System Board"

#define	EISA_PRODUCT_MCY2501	"Microdyne NE2500/NE5500 Series Ethernet"

#define	EISA_PRODUCT_MET1104	"Metheus UGA 1104 Graphics"
#define	EISA_PRODUCT_MET1128	"Metheus UGA 1124/1128 Graphics"

#define	EISA_PRODUCT_MIC0001	"Micronics 486 66/50/33/25 System Board"
#define	EISA_PRODUCT_MIC0004	"Micronics 486 PCI-EISA System Board"
#define	EISA_PRODUCT_MIC0005	"Micronics M5PE EISA-PCI Pentium System Board"
#define	EISA_PRODUCT_MIC0021	"Micronics NET3200 Ethernet LAN Adapter"
#define	EISA_PRODUCT_MIC0054	"Micronics M54Pe Dual Pentium PCI-EISA System Board"
#define	EISA_PRODUCT_MIC3001	"Micronics EISA3 System Board"
#define	EISA_PRODUCT_MIC5402	"Micronics M54E2 Dual Pentium PCI-EISA System Board"

/*
MINIADP	Adaptec 32-bit SCSI Host Adapter (with floppy)
*/

#define	EISA_PRODUCT_MIR0928	"miroCRYSTAL / miroMAGIC / miroRAINBOW (14-Sep-93)"

/* Mylex Products */
#define	EISA_PRODUCT_MLX0010	"Mylex LNE390A 32-bit Ethernet"
#define	EISA_PRODUCT_MLX0011	"Mylex LNE390B 32-bit Ethernet"
#define	EISA_PRODUCT_MLX0020	"Mylex DCE376 32-Bit SCSI"
#define	EISA_PRODUCT_MLX0021	"Mylex DCE376 32-Bit SCSI"
#define	EISA_PRODUCT_MLX0022	"Mylex DCE376 32-Bit SCSI"
#define	EISA_PRODUCT_MLX0030	"Mylex LNI390A ISA 16-Bit Ethernet"
#define	EISA_PRODUCT_MLX0040	"Mylex GXE020B/C 32-Bit Graphics"
#define	EISA_PRODUCT_MLX0050	"Mylex GLE(911) Graphics"
#define	EISA_PRODUCT_MLX0070	"Mylex DAC960 Disk Array"
#define	EISA_PRODUCT_MLX0071	"Mylex DAC960 Disk Array (3-channel)"
#define	EISA_PRODUCT_MLX0072	"Mylex DAC960 Disk Array (3-channel)"
#define	EISA_PRODUCT_MLX0073	"Mylex DAC960 Disk Array (2-channel)"
#define	EISA_PRODUCT_MLX0074	"Mylex DAC960 Disk Array (1-channel)"
#define	EISA_PRODUCT_MLX0075	"Mylex DAC960-A Disk Array (3-channel)"
#define	EISA_PRODUCT_MLX0076	"Mylex DAC960-A Disk Array (2-channel)"
#define	EISA_PRODUCT_MLX0077	"Mylex DAC960-A Disk Array (1-channel)"
#define	EISA_PRODUCT_MLX0101	"Mylex LME596 32-bit 4 Channel Ethernet"
#define	EISA_PRODUCT_MLXFD01	"Mylex MDE486 32-Bit 486 System Board"
#define	EISA_PRODUCT_MLXFE01	"Mylex MBE486 32-Bit 486 System Board"
#define	EISA_PRODUCT_MLXFF01	"Mylex MAE486 32-Bit 486 System Board"
#define	EISA_PRODUCT_MLXFF02	"Mylex MDE486/MNE486 32-Bit 486 System Board"

/* Matrox products */
#define	EISA_PRODUCT_MTX2040	"Matrox IM-1280/EISA"

/* NEC products */
#define	EISA_PRODUCT_NEC8201	"NEC DPT SCSI Host Bus Adapter w/ Cache (PM2012B/90)"

/* National Instruments Corp. products */
#define	EISA_PRODUCT_NIC0202	"National Instruments Corp. AT-MIO-16 Multi-function Board"
#define	EISA_PRODUCT_NIC0301	"National Instruments Corp. AT-DIO-32F Digital I/O Board"
#define	EISA_PRODUCT_NIC0400	"National Instruments Corp. PC-DIO-24 Digital I/O Board"
#define	EISA_PRODUCT_NIC0501	"National Instruments Corp. LAB-PC/LAB-PC+ Multi-function Board"
#define	EISA_PRODUCT_NIC0602	"National Instruments Corp. AT-MIO-16F-5 Multi-function Board"
#define	EISA_PRODUCT_NIC0700	"National Instruments Corp. PC-DIO-96 Digital I/O Board"
#define	EISA_PRODUCT_NIC0800	"National Instruments Corp. PC-LPM-16 Low Power Multi-function Board"
#define	EISA_PRODUCT_NIC0900	"National Instruments Corp. PC-TIO-10 Timing I/O Board"
#define	EISA_PRODUCT_NIC1000	"National Instruments Corp. AT-A2150 16-bit 4 Channel A/D Board"
#define	EISA_PRODUCT_NIC1100	"National Instruments Corp. AT-DSP2200 DSP Accelerator/Audio I/O Board"
#define	EISA_PRODUCT_NIC1200	"National Instruments Corp. AT-AO-6/10 ANALOG OUTPUT BOARD"
#define	EISA_PRODUCT_NIC1300	"National Instruments Corp. AT-MIO-16X Multi-function Board"
#define	EISA_PRODUCT_NIC1400	"National Instruments Corp. AT-MIO-64F-5 Multi-function Board"
#define	EISA_PRODUCT_NIC1500	"National Instruments Corp. AT-MIO-16D Multi-function Board"
#define	EISA_PRODUCT_NICC005	"National Instruments Corp. AT-GPIB Interface Board"
#define	EISA_PRODUCT_NICC105	"National Instruments Corp. GPIB-PCIIA Interface Board"
#define	EISA_PRODUCT_NICC205	"National Instruments Corp. GPIB-PCII Interface Board"
#define	EISA_PRODUCT_NICC304	"National Instruments Corp. AT-GPIB/TNT"
#define	EISA_PRODUCT_NICC502	"National Instruments Corp. EISA-GPIB"

/* c't Universal products */
#define	EISA_PRODUCT_NON0101	"c't Universal 16-Bit Multi I/O"
#define	EISA_PRODUCT_NON0102	"c't Universal 16-Bit Multi I/O"
#define	EISA_PRODUCT_NON0201	"c't Universal 8-Bit Multi I/O"
#define	EISA_PRODUCT_NON0301	"c't Universal Graphics"
#define	EISA_PRODUCT_NON0401	"c't Universal Ethernet"
#define	EISA_PRODUCT_NON0501	"c't Universal 16-Bit Sound"
#define	EISA_PRODUCT_NON0601	"c't Universal 8-Bit Sound"

/* Newport Systems Solutions products */
#define	EISA_PRODUCT_NSS0011	"Newport Systems Solutions WNIC Adapter"

/* Novell products */
#define	EISA_PRODUCT_NVL0701	"Novell NE3200 Bus Master Ethernet"
#define	EISA_PRODUCT_NVL0901	"Novell NE2100 Ethernet/Cheapernet"
#define	EISA_PRODUCT_NVL1001	"Novell NMSL (Netware Mirrored Server Link)"
#define	EISA_PRODUCT_NVL1201	"Novell NE32HUB 32-bit Base"
#define	EISA_PRODUCT_NVL1301	"Novell NE32HUB 32-bit TPE"
#define	EISA_PRODUCT_NVL1401	"Novell NE32HUB PME ISA"
#define	EISA_PRODUCT_NVL1501	"Novell NE2000PLUS Ethernet"

/* Olicom products */
#define	EISA_PRODUCT_OLC0701	"Olicom ISA 16/4 Token-Ring"
#define	EISA_PRODUCT_OLC0702	"Olicom OC-3117, ISA 16/4 Token-Ring"
#define	EISA_PRODUCT_OLC0801	"Olicom OC-3118 ISA 16/4 Token-Ring"
#define	EISA_PRODUCT_OLC0901	"Olicom 16/4 Token-Ring"
#define	EISA_PRODUCT_OLC0902	"Olicom 16/4 Token-Ring"
#define	EISA_PRODUCT_OLC0B01	"Olicom PCA 16/4 Token-Ring"
#define	EISA_PRODUCT_OLC1201	"Olicom 32 Bit 16/4 Token-Ring"

/* OPTi products */
#define	EISA_PRODUCT_OPT0000	"OPTi HUNTER 32-Bit 486 System Board"
#define	EISA_PRODUCT_OPT0200	"OPTi LOW_COST 32-Bit 486 System Board"

#define	EISA_PRODUCT_OTI0011	"Pro II/EISA"

#define	EISA_PRODUCT_PCI0080	"Pioneer 486WB 8 SLOT SYSTEM BOARD"
#define	EISA_PRODUCT_PCI0120	"Pioneer 486WB 12 SLOT SYSTEM BOARD"
#define	EISA_PRODUCT_PCI2080	"Pioneer 486WB 8 SLOT SYSTEM BOARD"

#define	EISA_PRODUCT_PHI8041	"Standard VGA controller"

#define	EISA_PRODUCT_PLX1001	"OCEAN EISALink 32-Bit BUS-MASTER Ethernet"

/* Proteon products */
#define	EISA_PRODUCT_PRO6000	"Proteon ProNET 4/16 Token Ring"
#define	EISA_PRODUCT_PRO6001	"Proteon ProNET 4/16 Token Ring"
#define	EISA_PRODUCT_PRO6002	"Proteon ProNET 4/16 Token Ring"

/* Poseidon products */
#define	EISA_PRODUCT_PTI5401	"Poseidon P6 QUAD PCI-EISA Board"

#define	EISA_PRODUCT_RII0101	"Racal InterLan ES3210 Ethernet"

#define	EISA_PRODUCT_SEC0010	"SAMSUNG ISA Multifunction Card"
#define	EISA_PRODUCT_SEC0020	"SAMSUNG VGA Card (GTI VC-004)"
#define	EISA_PRODUCT_SEC0021	"WD 90C31 Local Bus VGA"
#define	EISA_PRODUCT_SECFF01	"SAMSUNG MAE486 System Board"
#define	EISA_PRODUCT_SECFF02	"OPTi/EISA 32-Bit 486 System Board"

#define	EISA_PRODUCT_SGT0101	"AT&T GIS 8/16 Port Serial Controller"

#define	EISA_PRODUCT_SIS0000	"4DESD EISA-486 System Board"
#define	EISA_PRODUCT_SIS0001	"EISA-486 Demo Board"

#define	EISA_PRODUCT_SKD0100	"SK-NET FDDI-FE 32-Bit"

/* Standard Microsystems (SMC) */
#define	EISA_PRODUCT_SMC0110	"Standard Microsystems Corp. Elite32 Ethernet"
#define	EISA_PRODUCT_SMC03E0	"Standard Microsystems Corp. EtherCard PLUS"
#define	EISA_PRODUCT_SMC13E0	"Standard Microsystems Corp. EtherCard PLUS Elite16"
#define	EISA_PRODUCT_SMC8003	"Standard Microsystems Corp. EtherCard PLUS Elite"
#define	EISA_PRODUCT_SMC8010	"Standard Microsystems Corp. Ethercard Elite32C Ultra"
#define	EISA_PRODUCT_SMC8013	"Standard Microsystems Corp. EtherCard PLUS Elite16"
#define	EISA_PRODUCT_SMC8216	"Standard Microsystems Corp. EtherCard Elite16 ULTRA"
#define	EISA_PRODUCT_SMCA010	"Standard Microsystems Corp. Ether 10/100"

/* Siemens Nixdorf products */
#define	EISA_PRODUCT_SNIAAA0	"Siemens Nixdorf Ether-Board EISA T"
#define	EISA_PRODUCT_SNIAAB0	"Siemens Nixdorf Ether-Board EISA 2"
#define	EISA_PRODUCT_SNIABF0	"Siemens Nixdorf ETHER-BOARD-AT105T Ethernet/Cheapernet"
#define	EISA_PRODUCT_SNIACA0	"Siemens Nixdorf ETHER-BOARD-AT10T Ethernet/Twisted Pair"

/* Stallion Technologies products */
#define	EISA_PRODUCT_STL0100	"Stallion Technologies ONboard/E"
#define	EISA_PRODUCT_STL0120	"Stallion Technologies ONboard ISA"
#define	EISA_PRODUCT_STL0130	"Stallion Technologies Brumby ISA"
#define	EISA_PRODUCT_STL0200	"Stallion Technologies EasyIO"
#define	EISA_PRODUCT_STL0400	"Stallion Technologies EC 8/64-EI"
#define	EISA_PRODUCT_STL0410	"Stallion Technologies EC 8/32-AT"
#define	EISA_PRODUCT_STL0420	"Stallion Technologies EC 8/64-AT"

#define	EISA_PRODUCT_SUK1022	"SK-NET Token Ring"
#define	EISA_PRODUCT_SUK1059	"SK-NET G16 Ethernet"
#define	EISA_PRODUCT_SUK1072	"SK-FDDI"

/* Thomas-Conrad products */
#define	EISA_PRODUCT_TCC010C	"Thomas-Conrad TC6045 ARC-Card/AT"
#define	EISA_PRODUCT_TCC030D	"Thomas-Conrad TC6042 ARC-Card/CE"
#define	EISA_PRODUCT_TCC040B	"Thomas-Conrad TC6142 ARC-Card/CE"
#define	EISA_PRODUCT_TCC3047	"Thomas-Conrad TC3047 TCNS Adapter/EISA"

/* 3Com products */
#define	EISA_PRODUCT_TCM3190	"3Com 3C319 TokenLink Velocity"
#define	EISA_PRODUCT_TCM5030	"3Com EtherLink II Family"
#define	EISA_PRODUCT_TCM5070	"3Com 3C507 Etherlink 16 or TP v2.0"
#define	EISA_PRODUCT_TCM5090	"3Com 3C509-TP Ethernet"
#define	EISA_PRODUCT_TCM5091	"3Com 3C509 Ethernet"
#define	EISA_PRODUCT_TCM5092	"3Com 3C579-TP Ethernet"
#define	EISA_PRODUCT_TCM5093	"3Com 3C579 Ethernet"
#define	EISA_PRODUCT_TCM5094	"3Com 3C509-Combo Ethernet"
#define	EISA_PRODUCT_TCM5095	"3Com 3C509-TPO Ethernet"
#define	EISA_PRODUCT_TCM5098	"3Com 3C509-TP Ethernet"
#define	EISA_PRODUCT_TCM5920	"3Com 3C592 Etherlink III"
#define	EISA_PRODUCT_TCM5970	"3Com 3C597 Fast Etherlink TX"
#define	EISA_PRODUCT_TCM5971	"3Com 3C597 Fast Etherlink T4"
#define	EISA_PRODUCT_TCM5972	"3Com 3C597 Fast Etherlink MII"
#define	EISA_PRODUCT_TCM7700	"3Com 3C770 FDDI"

/* Thomas-Conrad products */
#define	EISA_PRODUCT_TCO010C	"Thomas-Conrad TC6045 ARC-Card/AT"
#define	EISA_PRODUCT_TCO030D	"Thomas-Conrad TC6042 ARC-Card/CE"
#define	EISA_PRODUCT_TCO040B	"Thomas-Conrad TC6142 ARC-Card/CE"
#define	EISA_PRODUCT_TCO050D	"Thomas-Conrad TC4035 TOKEN RING Adapter/AT (Rev D)"
#define	EISA_PRODUCT_TCO3147	"Thomas-Conrad TC3047 System (TCNS) Adapter"
#define	EISA_PRODUCT_TCO345A	"Thomas-Conrad TC3045 Network System (TCNS) AT Adapter"
#define	EISA_PRODUCT_TCO345B	"Thomas-Conrad TC3045 Network System (TCNS) AT Adapter"

/* Tecmar products */
#define	EISA_PRODUCT_TEC8000	"Tecmar QIC60 HOST ADAPTER"
#define	EISA_PRODUCT_TEC8001	"Tecmar QIC PC36 TAPE CONTROLLER"
#define	EISA_PRODUCT_TEC8002	"Tecmar QT HOST ADAPTER"
#define	EISA_PRODUCT_TEC8003	"Tecmar QT PC36 TAPE CONTROLLER"

#define	EISA_PRODUCT_TRM0001	"EISA-486C SYSTEM BOARD"
#define	EISA_PRODUCT_TRM0320	"DC-320 SCSI Host Adapter"
#define	EISA_PRODUCT_TRM0620	"DC-620 IDE Cache Controller"
#define	EISA_PRODUCT_TRM0820	"DC-820 SCSI Cache Host Adapter"
#define	EISA_PRODUCT_TRM320E	"DC-320E SCSI Host Adapter"
#define	EISA_PRODUCT_TRM820B	"DC-820B SCSI Cache Host Adapter"

/* Truevision products  */
#define	EISA_PRODUCT_TRU0210	"Truevision Image Capture Board"
#define	EISA_PRODUCT_TRU0520	"Truevision ATVista (R) VideoGraphics"
#define	EISA_PRODUCT_TRU1100	"Truevision DVR"

/* Texas Instruments products  */
#define	EISA_PRODUCT_TXN0011	"Texas Instruments TACT84500 MODULAR SYSTEM BOARD"

/* Tyan products */
#define	EISA_PRODUCT_TYN0000	"Tyan 486 PRO-EISA Board"
#define	EISA_PRODUCT_TYN0001	"Tyan VL EISA-486 Board"
#define	EISA_PRODUCT_TYN0003	"Tyan S1452/S1462 PCI-EISA Main Board"

/* Ungermann-Bass products */
#define	EISA_PRODUCT_UBIA100	"Ungermann-Bass Personal NIU"
#define	EISA_PRODUCT_UBIA200	"Ungermann-Bass Personal NIU/ex"
#define	EISA_PRODUCT_UBIB100	"Ungermann-Bass NIUpc"
#define	EISA_PRODUCT_UBIB200	"Ungermann-Bass 3270 NIUpc"
#define	EISA_PRODUCT_UBIC100	"Ungermann-Bass NIC"
#define	EISA_PRODUCT_UBID100	"Ungermann-Bass NIUpc/Token Ring"

/* UltraStor products */
#define	EISA_PRODUCT_USC0120	"UltraStor 12f ISA ESDI"
#define	EISA_PRODUCT_USC0125	"UltraStor 12c ESDI"
#define	EISA_PRODUCT_USC0140	"UltraStor 14f ISA SCSI"
#define	EISA_PRODUCT_USC0220	"UltraStor U22c"
#define	EISA_PRODUCT_USC0225	"UltraStor 22f ESDI"
#define	EISA_PRODUCT_USC0240	"UltraStor 24f SCSI"
#define	EISA_PRODUCT_USC0340	"UltraStor 34f VESA VL-BUS"

/* US Robotics products */
#define	EISA_PRODUCT_USR0011	"US Robotics 33.6 Telephony Modem"
#define	EISA_PRODUCT_USR3401	"US Robotics V.34-Ready Fax Modem"

/* Vermont products */
#define	EISA_PRODUCT_VMI0201	"Vermont Image Manager 1024"
#define	EISA_PRODUCT_VMI0211	"Vermont Cobra"
#define	EISA_PRODUCT_VMI0601	"Vermont Image Manager 640"
#define	EISA_PRODUCT_VMI0E01	"Vermont Cobra Plus"

/* Western Digital products */
#define	EISA_PRODUCT_WDC0101	"Western Digital WD1009V-MM1 Winchester Controller"
#define	EISA_PRODUCT_WDC0102	"Western Digital WD1009V-SE1 Winchester Controller"
#define	EISA_PRODUCT_WDC0300	"Western Digital StarCard PLUS 8003S"
#define	EISA_PRODUCT_WDC0301	"Western Digital StarLink PLUS 8003SH"
#define	EISA_PRODUCT_WDC03E0	"Western Digital EtherCard PLUS 8003E"
#define	EISA_PRODUCT_WDC03E1	"Western Digital EtherCard PLUS w/Boot 8003EBT"
#define	EISA_PRODUCT_WDC03E2	"Western Digital EtherCard + 8003EB 61-600245-02"
#define	EISA_PRODUCT_WDC03E3	"Western Digital EtherCard PLUS TP 8003WT"
#define	EISA_PRODUCT_WDC03E4	"Western Digital EtherCard + 8003EB 61-600090-00"
#define	EISA_PRODUCT_WDC0510	"Western Digital TokenCard 8005TR/8005TRWS"
#define	EISA_PRODUCT_WDC1009	"Western Digital WD1009V-MM1/MM2 Winchester Controller"
#define	EISA_PRODUCT_WDC13E0	"Western Digital EtherCard PLUS 16 8013EBT"

#define	EISA_PRODUCT_XTI02B1	"XTI XNET 1800 PARALLEL SWITCH"
