/* $OpenBSD: fq_codel.h,v 1.4 2020/06/18 23:29:59 dlg Exp $ */

/*
 * Copyright (c) 2017 Mike Belopuhov
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef _NET_FQ_CODEL_H_
#define _NET_FQ_CODEL_H_

/* compatible with hfsc_pktcntr */
struct fqcodel_pktcntr {
	uint64_t		packets;
	uint64_t		bytes;
};

struct fqcodel_stats {
	struct fqcodel_pktcntr 	xmit_cnt;
	struct fqcodel_pktcntr 	drop_cnt;

	uint32_t		qlength;
	uint32_t		qlimit;

	uint32_t		flows;
	uint32_t		_unused;   /* padding */

	uint32_t		target;
	uint32_t		interval;

	/* the values below are used to calculate standard deviation */
	uint64_t		delaysum;  /* sum of delays, us */
	uint64_t		delaysumsq;/* sum of squared delays, us */
};

#ifdef _KERNEL
extern const struct ifq_ops * const ifq_fqcodel_ops;
extern const struct pfq_ops * const pfq_fqcodel_ops;
#endif	/* _KERNEL */

#endif	/* _NET_FQ_CODEL_H_ */
